#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the DetectOS module."""

from __future__ import absolute_import
import unittest
from mock import patch, mock_open, MagicMock as Mock
from ILCDIRAC.Core.Utilities.DetectOS import NativeMachine
from Tests.Utilities.GeneralUtils import assertEqualsImproved

__RCSID__ = "$Id$"
MODULE_NAME = 'ILCDIRAC.Core.Utilities.DetectOS'


class TestNativeMachine(unittest.TestCase):  # pylint: disable=too-many-public-methods
  """Tests the NativeMachine class."""

  # pylint: disable=protected-access
  def test_constructor(self):
    with patch('%s.platform.uname' % MODULE_NAME, new=Mock(return_value=['TestOS', 'hostname', 'kernel version', 'date', 'x86_64'])):
      mach_unix = NativeMachine()
    mach_win32 = get_win32_machine()
    mach_win64 = get_win64_machine()
    with patch('%s.platform.uname' % MODULE_NAME, new=Mock(return_value=['Linux', 'hostname', 'kernel version', 'date', 'x86_64'])):
      mach_linux_64bit = NativeMachine()
    with patch('%s.platform.uname' % MODULE_NAME, new=Mock(return_value=['Linux', 'hostname', 'kernel version', 'date', 'i686'])):
      mach_linux_32bit = NativeMachine()
    os_mock = Mock()
    os_mock.read.return_value = 'powerpc1'
    with patch('%s.platform.uname' % MODULE_NAME, new=Mock(return_value=['Darwin', 'hostname', 'kernel version', 'date', 'i686'])), \
         patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=os_mock)):
      mach_darwin_PowerPC = NativeMachine()

    assertEqualsImproved(mach_unix._sysinfo, ['TestOS', 'hostname',
                                               'kernel version', 'date',
                                               'x86_64'], self)
    self.assertIsNone(mach_unix._arch)
    assertEqualsImproved(mach_unix._ostype, 'TestOS', self)
    assertEqualsImproved(mach_unix._machine, 'x86_64', self)

    assertEqualsImproved(mach_win32._sysinfo, ['mysystem'], self)
    assertEqualsImproved(mach_win32._arch, '32', self)
    assertEqualsImproved(mach_win32._ostype, 'Windows', self)
    assertEqualsImproved(mach_win32._machine, 'i686', self)

    self.assertIsNone(mach_win64._sysinfo)
    assertEqualsImproved(mach_win64._arch, '64', self)
    assertEqualsImproved(mach_win64._ostype, 'Windows', self)
    assertEqualsImproved(mach_win64._machine, 'x86_64', self)

    assertEqualsImproved(mach_linux_64bit._sysinfo, ['Linux', 'hostname',
                                                      'kernel version', 'date',
                                                      'x86_64'], self)
    assertEqualsImproved(mach_linux_64bit._arch, '64', self)
    assertEqualsImproved(mach_linux_64bit._ostype, 'Linux', self)
    assertEqualsImproved(mach_linux_64bit._machine, 'x86_64', self)

    assertEqualsImproved(mach_linux_32bit._sysinfo, ['Linux', 'hostname',
                                                      'kernel version', 'date',
                                                      'i686'], self)
    assertEqualsImproved(mach_linux_32bit._arch, '32', self)
    assertEqualsImproved(mach_linux_32bit._ostype, 'Linux', self)
    assertEqualsImproved(mach_linux_32bit._machine, 'i686', self)

    assertEqualsImproved(mach_darwin_PowerPC._sysinfo, ['Darwin', 'hostname',
                                                         'kernel version', 'date',
                                                         'i686'], self)
    assertEqualsImproved(mach_darwin_PowerPC._arch, 'ppc', self)
    assertEqualsImproved(mach_darwin_PowerPC._ostype, 'Darwin', self)
    assertEqualsImproved(mach_darwin_PowerPC._machine, 'i686', self)

  def test_osflavor_win(self):
    nm = get_win32_machine()
    nm._sysinfo = ('Windows', 'dhellmann', '2008ServerR2', '6.1.7600', 'AMD64',
                   'Intel64 Family 6 Model 15 Stepping 11, GenuineIntel')
    result = nm.OSFlavour()
    assertEqualsImproved(result, '2008ServerR2', self)
    assertEqualsImproved(nm._osflavor, '2008ServerR2', self)

  def test_osflavor_sun(self):
    nm = get_win32_machine()
    nm._ostype = 'SunOS'
    result = nm.OSFlavour()
    assertEqualsImproved(result, 'sun', self)
    assertEqualsImproved(nm._osflavor, 'sun', self)
    assertEqualsImproved(nm._osversion, '4.x', self)

  def test_osflavor_darwin_1(self):
    nm = get_win32_machine()
    nm._ostype = 'Darwin'
    nm._sysinfo = ('Darwin', 'hubert.local', '7.4.2',
                    'Darwin Kernel Version 11.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
                    'x86_64', 'i386')
    result = nm.OSFlavour()
    assertEqualsImproved(result, 'Panther', self)
    assertEqualsImproved(nm._osflavor, 'Panther', self)
    assertEqualsImproved(nm._osversion, '10.3.4', self)

  def test_osflavor_darwin_2(self):
    nm = get_win32_machine()
    nm._ostype = 'Darwin'
    nm._sysinfo = ('Darwin', 'hubert.local', '8.4.2',
                    'Darwin Kernel Version 11.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
                    'x86_64', 'i386')
    result = nm.OSFlavour()
    assertEqualsImproved(result, 'Tiger', self)
    assertEqualsImproved(nm._osflavor, 'Tiger', self)
    assertEqualsImproved(nm._osversion, '10.4.4', self)

  def test_osflavor_darwin_3(self):
    nm = get_win32_machine()
    nm._ostype = 'Darwin'
    nm._sysinfo = ('Darwin', 'hubert.local', '9.4.2',
                    'Darwin Kernel Version 11.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
                    'x86_64', 'i386')
    result = nm.OSFlavour()
    assertEqualsImproved(result, 'Leopard', self)
    assertEqualsImproved(nm._osflavor, 'Leopard', self)
    assertEqualsImproved(nm._osversion, '10.5.4', self)

  def test_osflavor_darwin_4(self):
    nm = get_win32_machine()
    nm._ostype = 'Darwin'
    nm._sysinfo = ('Darwin', 'hubert.local', '10.4.2',
                    'Darwin Kernel Version 11.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
                    'x86_64', 'i386')
    result = nm.OSFlavour()
    assertEqualsImproved(result, 'Snow Leopard', self)
    assertEqualsImproved(nm._osflavor, 'Snow Leopard', self)
    assertEqualsImproved(nm._osversion, '10.6.4', self)

  def test_osflavor_darwin_unknown(self):
    nm = get_win32_machine()
    nm._ostype = 'Darwin'
    nm._sysinfo = ('Darwin', 'hubert.local', '11.4.2',
                    'Darwin Kernel Version 11.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
                    'x86_64', 'i386')
    result = nm.OSFlavour()
    assertEqualsImproved(result, None, self)
    assertEqualsImproved(nm._osflavor, None, self)
    assertEqualsImproved(nm._osversion, '10.7.4', self)

  def test_osflavor_misc(self):
    nm = get_win32_machine()
    nm._ostype = 'misc'
    self.assertIsNone(nm.OSFlavour())
    self.assertIsNone(nm._osflavor)

  def test_osflavor_unix(self):
    nm = NativeMachine()
    nm._osflavor = None
    nm._ostype = 'Linux'
    file_content = 'something\nSuSE 10.2\nsomething'
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=[False, True])), \
         patch('%s.open' % MODULE_NAME, mock_open(read_data=file_content), create=True):
      result = nm.OSFlavour()  # Extracts version number from 'MyCoolOS 20.12' strings
      assertEqualsImproved(result, 'SuSE', self)
      assertEqualsImproved(nm._osflavor, 'SuSE', self)
      assertEqualsImproved(nm._osversion, '10.2', self)

  def test_osflavor_unix_with_teststr(self):
    nm = NativeMachine()
    nm._osflavor = None
    nm._ostype = 'Linux'
    file_content = ''
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=[False, True])), \
         patch('%s.open' % MODULE_NAME, mock_open(read_data=file_content), create=True):
      result = nm.OSFlavour('ubunTu 13.21')
      assertEqualsImproved(result, 'Ubuntu', self)
      assertEqualsImproved(nm._osflavor, 'Ubuntu', self)
      assertEqualsImproved(nm._osversion, '13.21', self)

  def test_osversion_win(self):
    nm = get_win32_machine()
    nm._sysinfo = ('Windows', 'dhellmann', '2008ServerR2', '6.1.7600', 'AMD64',
                     'Intel64 Family 6 Model 15 Stepping 11, GenuineIntel')
    result = nm.OSVersion(3)
    assertEqualsImproved(result, '6.1.7600', self)
    assertEqualsImproved(nm._osversion, '6.1.7600', self)

  def test_osversion_win_position_0(self):
    nm = get_win32_machine()
    nm._sysinfo = ('Windows', 'dhellmann', '2008ServerR2', '6.1.7600', 'AMD64',
                     'Intel64 Family 6 Model 15 Stepping 11, GenuineIntel')
    result = nm.OSVersion(0)
    assertEqualsImproved(result, '6.1.7600', self)  # `if position` ignores pos=0
    assertEqualsImproved(nm._osversion, '6.1.7600', self)

  def test_osversion_win_position_1(self):
    nm = get_win32_machine()
    nm._sysinfo = ('Windows', 'dhellmann', '2008ServerR2', '6.1.7600', 'AMD64',
                     'Intel64 Family 6 Model 15 Stepping 11, GenuineIntel')
    result = nm.OSVersion(1)
    assertEqualsImproved(result, '6', self)
    assertEqualsImproved(nm._osversion, '6.1.7600', self)

  def test_osversion_win_position_2(self):
    nm = get_win32_machine()
    nm._sysinfo = ('Windows', 'dhellmann', '2008ServerR2', '6.1.7600', 'AMD64',
                     'Intel64 Family 6 Model 15 Stepping 11, GenuineIntel')
    result = nm.OSVersion(2)
    assertEqualsImproved(result, '6.1', self)
    assertEqualsImproved(nm._osversion, '6.1.7600', self)

  def test_osversion_win_position_4(self):
    nm = get_win32_machine()
    nm._sysinfo = ('Windows', 'dhellmann', '2008ServerR2', '6.1.7600', 'AMD64',
                     'Intel64 Family 6 Model 15 Stepping 11, GenuineIntel')
    result = nm.OSVersion(4)
    assertEqualsImproved(result, '6.1.7600', self)
    assertEqualsImproved(nm._osversion, '6.1.7600', self)

  def test_osversion_sun(self):
    nm = get_win32_machine()
    nm._ostype = 'SunOS'
    result = nm.OSVersion(2)
    assertEqualsImproved(result, '4.x', self)
    assertEqualsImproved(nm._osversion, '4.x', self)

  def test_osversion_sun_position(self):
    nm = get_win32_machine()
    nm._ostype = 'SunOS'
    result = nm.OSVersion(1)
    assertEqualsImproved(result, '4', self)
    assertEqualsImproved(nm._osversion, '4.x', self)

  def test_osversion_darwin(self):
    mach = get_naked_machine()
    mach._ostype = 'Darwin'
    mach._sysinfo = (
        'Darwin',
        'hubert.local',
        '11.4.2',
        'Darwin Kernel Version 11.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
        'x86_64',
        'i386')
    result = mach.OSVersion(3)
    assertEqualsImproved(result, '10.7.4', self)
    assertEqualsImproved(mach._osversion, '10.7.4', self)

  def test_osversion_linux(self):
    mach = get_naked_machine()
    mach._osversion = None
    mach._ostype = 'Linux'
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.open' % MODULE_NAME, mock_open(read_data='somethingsomething\nUbuntu 19.3'), create=True):
      result = mach.OSVersion(5)
      assertEqualsImproved(result, '19.3', self)
      assertEqualsImproved(mach._osversion, '19.3', self)

  def test_osversion_linux_with_teststring(self):
    mach = get_naked_machine()
    mach._ostype = 'Linux'
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.open' % MODULE_NAME, mock_open(read_data='somethingsomething\nUbuntu 19.3'), create=True):
      result = mach.OSVersion(2, 'SuSE 3.6')
      assertEqualsImproved(result, '3.6', self)
      assertEqualsImproved(mach._osversion, '3.6', self)

  def test_osversion_already_set(self):
    mach = get_naked_machine()
    mach._osversion = '151.21.5'
    res_1 = mach.OSVersion()
    res_2 = mach.OSVersion(10000)
    res_3 = mach.OSVersion(2)
    assertEqualsImproved(res_1, '151.21.5', self)
    assertEqualsImproved(res_2, '151.21.5', self)
    assertEqualsImproved(res_3, '151.21', self)
    assertEqualsImproved(mach._osversion, '151.21.5', self)

  def test_osversion_no_match(self):
    mach = get_naked_machine()
    mach._ostype = 'Linux'
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.open' % MODULE_NAME, mock_open(read_data='somethingsomething\nUbuntu 19.3'), create=True):
      result = mach.OSVersion(teststring='wrong_file123')
      self.assertIsNone(result)
      self.assertIsNone(mach._osversion)

  def test_native_compiler_version_win(self):
    win32_mach = get_win32_machine()
    result = win32_mach.nativeCompilerVersion()
    assertEqualsImproved(result, 'vc71', self)
    assertEqualsImproved(win32_mach._compversion, 'vc71', self)

  def test_native_compiler_version_unix(self):
    unix_mach = get_win32_machine()
    unix_mach._ostype = 'Linux'
    proc_mock = Mock()
    proc_mock.readlines.return_value = [
        'something',
        'g++ (GCC) 4.4.7 20120313 (Red Hat 4.4.7-17)',
        'Copyright (C) 2010 Free Software Foundation, Inc.',
        'This is free software; see the source for copying conditions.  There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.']
    with patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=proc_mock)) as popen_mock:
      result = unix_mach.nativeCompilerVersion(2)
      assertEqualsImproved(result, '4.4', self)
      popen_mock.assert_called_once_with('g++ --version')
      assertEqualsImproved(unix_mach._compversion, '4.4.7', self)

  def test_native_compiler_version_unix_2(self):
    unix_mach = get_win32_machine()
    unix_mach._ostype = 'Linux'
    proc_mock = Mock()
    proc_mock.readlines.return_value = [
        'something',
        'g++ (GCC) 5.2.1 20120313 (Red Hat 5.2.1-17)',
        'Copyright (C) 2010 Free Software Foundation, Inc.',
        'This is free software; see the source for copying conditions.  There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.']
    with patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=proc_mock)) as popen_mock:
      result = unix_mach.nativeCompilerVersion(3)
      assertEqualsImproved(result, '5.2.1', self)
      popen_mock.assert_called_once_with('g++ --version')
      assertEqualsImproved(unix_mach._compversion, '5.2.1', self)

  def test_native_compiler_version_preset(self):
    unix_mach = get_win32_machine()
    unix_mach.nativeCompilerVersion()
    unix_mach._ostype = 'Linux'
    proc_mock = Mock()
    proc_mock.readlines.return_value = [
        'something',
        'g++ (GCC) 4.4.7 20120313 (Red Hat 4.4.7-17)',
        'Copyright (C) 2010 Free Software Foundation, Inc.',
        'This is free software; see the source for copying conditions.  There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.']
    with patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=proc_mock)):
      result = unix_mach.nativeCompilerVersion(2)
      assertEqualsImproved(result, 'vc71', self)
      assertEqualsImproved(unix_mach._compversion, 'vc71', self)

  def test_native_compiler_version_nomatch(self):
    unix_mach = get_win32_machine()
    unix_mach._ostype = 'Linux'
    proc_mock = Mock()
    proc_mock.readlines.return_value = ['Malformed_Version_String']
    with patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=proc_mock)):
      result = unix_mach.nativeCompilerVersion()
      self.assertIsNone(result)
      self.assertIsNone(unix_mach._compversion)

  def test_native_compiler_win(self):
    win_mach = get_win32_machine()
    result = win_mach.nativeCompiler()
    assertEqualsImproved(result, 'vc71', self)
    assertEqualsImproved(win_mach._compiler, 'vc71', self)

  def test_native_compiler_preset(self):
    win_mach = get_win32_machine()
    win_mach.nativeCompiler()
    win_mach._ostype = 'Linux'
    result = win_mach.nativeCompiler()
    assertEqualsImproved(result, 'vc71', self)
    assertEqualsImproved(win_mach._compiler, 'vc71', self)

  def test_native_compiler_unix(self):
    unix_mach = get_win32_machine()
    unix_mach._ostype = 'Linux'
    proc_mock = Mock()
    proc_mock.readlines.return_value = ['g++ (GCC) 4.4.7 20120313 (Red Hat 4.4.7-17)']
    with patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=proc_mock)):
      result = unix_mach.nativeCompiler()
      assertEqualsImproved(result, 'gcc44', self)
      assertEqualsImproved(unix_mach._compiler, 'gcc44', self)

  def test_native_compiler_unix_v3(self):
    unix_mach = get_win32_machine()
    unix_mach._ostype = 'Linux'
    proc_mock = Mock()
    proc_mock.readlines.return_value = ['g++ (GCC) 3.3.7 20120313 (Red Hat 3.3.7-17)']
    with patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=proc_mock)):
      result = unix_mach.nativeCompiler()
      assertEqualsImproved(result, 'gcc337', self)
      assertEqualsImproved(unix_mach._compiler, 'gcc337', self)

  def test_native_compiler_unix_v34(self):
    unix_mach = get_win32_machine()
    unix_mach._ostype = 'Linux'
    proc_mock = Mock()
    proc_mock.readlines.return_value = ['g++ (GCC) 3.4.7 20120313 (Red Hat 3.4.7-17)']
    with patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=proc_mock)):
      result = unix_mach.nativeCompiler()
      assertEqualsImproved(result, 'gcc34', self)
      assertEqualsImproved(unix_mach._compiler, 'gcc34', self)

  def test_native_compiler_darwin(self):
    unix_mach = get_win32_machine()
    unix_mach._ostype = 'Darwin'
    unix_mach._sysinfo = (
        'Darwin',
        'hubert.local',
        '9.4.2',
        'Darwin Kernel Version 11.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
        'x86_64',
        'i386')
    proc_mock = Mock()
    proc_mock.readlines.return_value = ['g++ (GCC) 6.3.1 20120313 (Red Hat 6.3.1-17)']
    with patch('%s.os.popen' % MODULE_NAME, new=Mock(return_value=proc_mock)):
      result = unix_mach.nativeCompiler()
      assertEqualsImproved(result, 'gcc631', self)
      assertEqualsImproved(unix_mach._compiler, 'gcc631', self)

  def test_cmt_architecture(self):
    unix_mach = get_naked_machine()
    unix_mach._machine = 'i686'
    res_1 = unix_mach.CMTArchitecture()
    unix_mach._machine = 'x86_64'
    res_2 = unix_mach.CMTArchitecture()
    unix_mach._machine = 'ia64'
    res_3 = unix_mach.CMTArchitecture()
    unix_mach._machine = 'Power MAC'
    res_4 = unix_mach.CMTArchitecture()
    unix_mach._ostype = 'Windows'
    unix_mach._machine = 'unknown'
    with patch('%s.sys.platform' % MODULE_NAME, new='mytestarchitecture'):
      res_5 = unix_mach.CMTArchitecture()
    unix_mach._ostype = 'test'
    res_6 = unix_mach.CMTArchitecture()

    assertEqualsImproved(res_1, 'ia32', self)
    assertEqualsImproved(res_2, 'amd64', self)
    assertEqualsImproved(res_3, 'ia64', self)
    assertEqualsImproved(res_4, 'ppc', self)
    assertEqualsImproved(res_5, 'mytestarchitecture', self)
    assertEqualsImproved(res_6, 'ia32', self)

  def test_cmt_system(self):
    unix_mach = get_naked_machine()
    unix_mach._ostype = 'Windows'
    res_1 = unix_mach.CMTSystem()
    unix_mach._ostype = 'Darwin'
    res_2 = unix_mach.CMTSystem()
    unix_mach._ostype = 'TestOS'
    unix_mach._machine = 'i386'
    res_3 = unix_mach.CMTSystem()
    unix_mach._machine = 'testArch'
    res_4 = unix_mach.CMTSystem()

    assertEqualsImproved(res_1, 'VisualC', self)
    assertEqualsImproved(res_2, 'Darwin-i386', self)
    assertEqualsImproved(res_3, 'TestOS-i386', self)
    assertEqualsImproved(res_4, 'TestOS-testArch', self)

  def test_cmt_osflavor(self):
    unix_mach = get_naked_machine()
    unix_mach._ostype = 'Windows'
    unix_mach._arch = 'mytestarch'
    res_1 = unix_mach.CMTOSFlavour()
    assertEqualsImproved(res_1, 'winmytestarch', self)
    unix_mach._ostype = 'Darwin'
    unix_mach._sysinfo = (
        'Darwin',
        'hubert.local',
        '11.4.2',
        'Darwin Kernel Version 11.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
        'x86_64',
        'i386')
    res_2 = unix_mach.CMTOSFlavour()
    assertEqualsImproved(res_2, 'osx107', self)
    unix_mach._ostype = 'Linux'
    unix_mach._osflavor = 'Ubuntu'
    unix_mach._osversion = '14.04'
    res_3 = unix_mach.CMTOSFlavour()
    assertEqualsImproved(res_3, 'ub14', self)
    unix_mach._ostype = 'Linux'
    unix_mach._osflavor = 'SuSE'
    unix_mach._osversion = '11.04'
    res_4 = unix_mach.CMTOSFlavour()
    assertEqualsImproved(res_4, 'suse11', self)
    unix_mach._osversion = None
    unix_mach._ostype = 'Linux'
    unix_mach._osflavor = 'Redhat'
    unix_mach._osversion = '17.07'
    res_5 = unix_mach.CMTOSFlavour()
    assertEqualsImproved(res_5, 'rh17.07', self)

  def test_cmt_osequivalent_flavor(self):
    unix_mach = get_naked_machine()
    unix_mach._ostype = 'Darwin'
    unix_mach._sysinfo = (
        'Darwin',
        'hubert.local',
        '9.4.2',
        'Darwin Kernel Version 9.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
        'x86_64',
        'i386')
    assertEqualsImproved(unix_mach.CMTOSEquivalentFlavour(), 'osx105', self)

  # def test_cmt_compatible_config( self ):
  # Uninteresting for unittests.

  def test_cmt_supported_config(self):
    unix_mach = get_naked_machine()
    unix_mach._machine = 'x86_64'
    unix_mach._ostype = 'Darwin'
    unix_mach._sysinfo = (
        'Darwin',
        'hubert.local',
        '9.4.2',
        'Darwin Kernel Version 9.4.2: Thu Aug 23 16:25:48 PDT 2012; root:xnu-1699.32.7~1/RELEASE_X86_64',
        'x86_64',
        'i386')
    assertEqualsImproved(unix_mach.CMTSupportedConfig(), [], self)

  def test_cmt_native_config(self):
    unix_mach = get_naked_machine()
    unix_mach._machine = 'x86_64'
    with patch('%s.NativeMachine.nativeCompiler' % MODULE_NAME, new=Mock(return_value='vc71')), patch('%s.NativeMachine.CMTOSFlavour' % MODULE_NAME, new=Mock(return_value='ub14')):
      result = unix_mach.CMTNativeConfig()
      assertEqualsImproved(result, 'x86_64-ub14-vc71-opt', self)


def get_win32_machine():
  win32_plat = Mock()
  win32_plat.uname.return_value = ['mysystem']
  # Example 'real' return value ('Windows', 'dhellmann', '2008ServerR2',
  # '6.1.7600', 'AMD64', 'Intel64 Family 6 Model 15 Stepping 11,
  # GenuineIntel')
  with patch('%s.platform' % MODULE_NAME, new=win32_plat), \
       patch('%s.sys.platform' % MODULE_NAME, new='win32'):
    mach_win32 = NativeMachine()
  return mach_win32


def get_win64_machine():
  with patch('%s.platform' % MODULE_NAME, new='abc'), \
      patch('%s.sys.platform' % MODULE_NAME, new='win64'):
    mach_win64 = NativeMachine()
  return mach_win64


def get_naked_machine():
  naked_plat = Mock()
  naked_plat.uname.return_value = ('SecretOS', None, None, None, None, None)
  with patch('%s.platform' % MODULE_NAME, new=naked_plat), \
       patch('%s.sys.platform' % MODULE_NAME, new='SecretOS'):
    mach_naked = NativeMachine()
  return mach_naked



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""helper functions for getting configuration options."""

from __future__ import absolute_import
import os


def getOptionValue(ops, basePath, optionName, defaultValue, levels):
  """get option from any place in the hierarchy starting from basepath, going through each section level.

  :param ops: Operation helper
  :param str basePath: section in Operation to start looking for the option
  :param str optionName: the name of the option to find
  :param defaultValue: the default value to use for this option
  :param list levels: the different [sub-]sub-sections to check for this option
  :returns: value at the deepest level in the configuration
  """

  join = os.path.join
  value = ops.getValue(join(basePath, optionName), defaultValue)

  path = basePath
  for level in levels:
    path = join(path, level)
    value = ops.getValue(join(path, optionName), value)

  return value



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Print version of current iLCDirac installation.

Mandatory when submitting support requests

.. seealso::

  :doc:`AdministratorGuide/CommandReference/dirac-version`, :doc:`UserGuide/CommandReference/Others/dirac-info` and how
  to submit a :ref:`support-request`.

.. versionadded:: 23.0

:author: A. Sailer
"""

from __future__ import print_function
from __future__ import absolute_import
import pprint
import os

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import gConfig
from DIRAC.Core.Security.ProxyInfo import getProxyInfo
from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getVOForGroup
from DIRAC.Core.Utilities.PrettyPrint import printTable

import ILCDIRAC

__RCSID__ = "$Id$"


@DIRACScript()
def main():
  """Print the ILCDIRAC and DIRAC versions and other information."""
  fields, records = getInfo()
  records.insert(0, ("ILCDirac Version:", ILCDIRAC.version))
  records.insert(1, ("DIRAC version:", DIRAC.version))

  printTable(fields, records, numbering=False)


def getInfo():
  """Retrieve information about setup, etc."""
  records = []

  records.append(('Setup', gConfig.getValue('/DIRAC/Setup', 'Unknown')))
  records.append(('ConfigurationServer', gConfig.getValue('/DIRAC/Configuration/Servers', [])))
  records.append(('Installation path', DIRAC.rootPath))

  if os.path.exists(os.path.join(DIRAC.rootPath, DIRAC.getPlatform(), 'bin', 'mysql')):
    records.append(('Installation type', 'server'))
  else:
    records.append(('Installation type', 'client'))

  records.append(('Platform', DIRAC.getPlatform()))

  ret = getProxyInfo(disableVOMS=False)
  if ret['OK']:
    print(pprint.pformat(ret))
    if 'group' in ret['Value']:
      vo = getVOForGroup(ret['Value']['group'])
    else:
      vo = getVOForGroup('')
    if not vo:
      vo = "None"
    records.append(('VirtualOrganization', vo))
    if 'identity' in ret['Value']:
      records.append(('User DN', ret['Value']['identity']))
    if 'secondsLeft' in ret['Value']:
      records.append(('Proxy validity, secs', {'Value': str(ret['Value']['secondsLeft']), 'Just': 'L'}))

  if gConfig.getValue('/DIRAC/Security/UseServerCertificate', True):
    records.append(('Use Server Certificate', 'Yes'))
  else:
    records.append(('Use Server Certificate', 'No'))
  if gConfig.getValue('/DIRAC/Security/SkipCAChecks', False):
    records.append(('Skip CA Checks', 'Yes'))
  else:
    records.append(('Skip CA Checks', 'No'))

  try:
    import gfalthr  # pylint: disable=import-error
    records.append(('gfal version', gfalthr.gfal_version()))
  except BaseException:
    pass

  fields = ['Option', 'Value']

  return fields, records


if __name__ == "__main__":
  main()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Module that converts stdhep files to slcio. Goes usually before Tomato.

:author: Ching Bon Lam
"""

from __future__ import absolute_import
import os

from DIRAC.Core.Utilities.Subprocess import shellCall
from DIRAC import S_ERROR, gLogger

from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Core.Utilities.PrepareLibs import removeLibc

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class StdHepConverter(ModuleBase):
  """Convert to SLCIO some stdhep files: Useful to run Tomato after Whizard."""

  def __init__(self):

    super(StdHepConverter, self).__init__()

    self.STEP_NUMBER = ''
    # Step parameters
    self.applicationName = 'StdhepConverter'

  def execute(self):
    """Called from Workflow."""
    # Get input variables

    self.result = self.resolveInputVariables()

    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')

    if not self.result['OK']:
      LOG.error('Failed to resolve input parameters:', self.result['Message'])
      return self.result

    if 'LCIO' not in os.environ:
      LOG.error("Environment variable LCIO was not defined, cannot do anything")
      return S_ERROR("Environment variable LCIO was not defined, cannot do anything")

    # removeLibc

    removeLibc(os.path.join(os.environ["LCIO"], "lib"))

    # Setting up script

    LD_LIBRARY_PATH = os.path.join("$LCIO", "lib")
    if 'LD_LIBRARY_PATH' in os.environ:
      LD_LIBRARY_PATH += ":" + os.environ['LD_LIBRARY_PATH']

    PATH = "$LCIO/bin"
    if 'PATH' in os.environ:
      PATH += ":" + os.environ['PATH']

    scriptContent = """
#!/bin/sh

################################################################################
# Dynamically generated script by StdHepConverter module                       #
################################################################################

declare -x LD_LIBRARY_PATH=%s
declare -x PATH=%s

for STDHEPFILE in *.stdhep; do
    stdhepjob $STDHEPFILE ${STDHEPFILE/.stdhep/.slcio} -1
done

exit $?

""" % (LD_LIBRARY_PATH, PATH)

    # Write script to file

    scriptPath = 'StdHepConverter_%s_Run_%s.sh' % (self.applicationVersion, self.STEP_NUMBER)

    if os.path.exists(scriptPath):
      os.remove(scriptPath)

    script = open(scriptPath, 'w')
    script.write(scriptContent)
    script.close()

    # Setup log file for application stdout

    if os.path.exists(self.applicationLog):
      os.remove(self.applicationLog)

    # Run code

    os.chmod(scriptPath, 0o755)

    command = 'sh -c "./%s"' % (scriptPath)

    self.setApplicationStatus('StdHepConverter %s step %s' % (self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''

    self.result = shellCall(0,
                             command,
                             callbackFunction=self.redirectLogOutput,
                             bufferLimit=20971520
                           )

    # Check results

    resultTuple = self.result['Value']
    status = resultTuple[0]

    return self.finalStatusReport(status)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the CalibrationAgent."""

from __future__ import absolute_import
from collections import defaultdict
import itertools
import unittest
import pytest
from mock import patch, MagicMock as Mock
from DIRAC import S_OK, S_ERROR
from ILCDIRAC.CalibrationSystem.Agent.CalibrationAgent import CalibrationAgent
from Tests.Utilities.GeneralUtils import assertInImproved, \
    assertEqualsImproved
from six.moves import range
from six.moves import zip

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.CalibrationSystem.Agent.CalibrationAgent'

# pylint: disable=protected-access,no-member


@pytest.fixture
def calibAgent(mocker):
  """Create calibration handler."""
  mocker.patch('DIRAC.Core.Base.Client.Client', new=Mock(return_value=S_OK()))
  mocker.patch('DIRAC.Core.Base.AgentModule.AgentModule.__init__', new=Mock(return_value=None))
  calibAgent = CalibrationAgent("dummyAgentName", "dummyLoadName")
  return calibAgent


def test_execute_calibrationWithNoDiracJobsIsPresentInRunningCalibrationList(mocker, calibAgent):
  """Test behaviour of the execute() function when there is a running calibration with no dirac jobs assigned to it.

  Calibration #63 is present in the list of calibrations but doesn't have any grid jobs associated to is.
  When considering calibration #63 the agent should send an error to the logger and continue with execution.
  """
  dict1 = {64: {1: 'Running', 2: 'Failed'},
           65: {5: 'Running', 6: 'Finished'},
           66: {14: 'Killed'}}
  dict2 = {64: {417251: 'Running', 12741: 'Failed'},
           65: {4178: 'Running', 444: 'Finished'},
           66: {555: 'Killed'}}
  dict3 = {64: {'Owner': 'ow1', 'OwnerGroup': 'owGr1', 'OwnerDN': 'dummy'},
           65: {'Owner': 'ow2', 'OwnerGroup': 'owGr2', 'OwnerDN': 'dummy'},
           66: {'Owner': 'ow3', 'OwnerGroup': 'owGr3', 'OwnerDN': 'dummy'}}
  mockReturn_fetchJobStatuses = S_OK({'jobStatusVsWorkerId': dict1,
                                      'jobStatusVsJobId': dict2, 'calibrationOwnership': dict3})

  mockReturn_getNumberOfJobsPerCalibration = S_OK({63: 666, 64: 2, 65: 2, 66: 1})
  mockReturn_getRunningCalibrations = S_OK([63, 64, 65, 66])

  mockReturn_calibrationService = Mock()
  mockReturn_calibrationService.getNumberOfJobsPerCalibration = (
      Mock(return_value=mockReturn_getNumberOfJobsPerCalibration))
  mockReturn_calibrationService.getRunningCalibrations = Mock(return_value=mockReturn_getRunningCalibrations)
  mockReturn_calibrationService.checkForStepIncrement = Mock(return_value=S_OK())
  calibAgent.calibrationService = mockReturn_calibrationService

  calibAgent.log = Mock()

  mocker.patch.object(calibAgent, "fetchJobStatuses", new=Mock(return_value=mockReturn_fetchJobStatuses))
  mocker.patch.object(calibAgent, "checkForCalibrationsToBeKilled", new=Mock(return_value=S_OK()))
  mock_requestResubmission = Mock(return_value=S_OK())
  mocker.patch.object(calibAgent, "requestResubmission", side_effect=mock_requestResubmission)
  res = calibAgent.execute()

  mock_requestResubmission.assert_called_once_with([(64, 2), (66, 14)])
  assert res['OK']


class CalibrationAgentTest(unittest.TestCase):
  """Test the implementation of the methods of the CalibrationAgent."""

  def setUp(self):
    """Set up the objects."""
    import DIRAC
    with patch.object(DIRAC.ConfigurationSystem.Client.PathFinder, 'getAgentSection', new=Mock(return_value='')):
      self.calag = CalibrationAgent('Calibration/testCalAgent', 'testLoadname')
    self.rpc_mock = Mock()
    with patch('%s.Client' % MODULE_NAME, new=self.rpc_mock):
      res = self.calag.initialize()
      assert res['OK']

  def test_getworkerid(self):
    """Get worker id."""
    assertEqualsImproved(self.calag._CalibrationAgent__getWorkerIDFromJobName(
        'PandoraCaloCalibration_calid_149814_workerid_813102'), 813102, self)

  def test_getcalibrationid(self):
    """Get calibration id."""
    assertEqualsImproved(self.calag._CalibrationAgent__getCalibrationIDFromJobName(
        'PandoraCaloCalibration_calid_149814_workerid_813102'), 149814, self)

  def test_getworkerid_splitfails(self):
    """Get worker id."""
    with self.assertRaises(IndexError):
      self.calag._CalibrationAgent__getWorkerIDFromJobName('PandoraCaloCalibrationcalid149814workerid813102')

  def test_getcalibrationid_splitfails(self):
    """Get calibration id."""
    with self.assertRaises(IndexError):
      self.calag._CalibrationAgent__getCalibrationIDFromJobName('PandoraCaloCalibrationcalid149814workerid813102')

  def test_getworkerid_noidgiven(self):
    """Get worker id."""
    with self.assertRaises(ValueError):
      self.calag._CalibrationAgent__getWorkerIDFromJobName('PandoraCaloCalibration_calid_149814_workerid_')

  def test_getcalibrationid_noidgiven(self):
    """Get calibration id."""
    with self.assertRaises(ValueError):
      self.calag._CalibrationAgent__getCalibrationIDFromJobName('PandoraCaloCalibration_calid_')

  def test_getworkerid_conversion_fails(self):
    """Get worker id."""
    with self.assertRaises(ValueError):
      assertEqualsImproved(self.calag._CalibrationAgent__getWorkerIDFromJobName(
          'PandoraCaloCalibration_calid_sixteen_workerid_twenty'), 813102, self)

  def test_getcalibrationid_conversion_fails(self):
    """Get calibration id."""
    with self.assertRaises(ValueError):
      assertEqualsImproved(self.calag._CalibrationAgent__getCalibrationIDFromJobName(
          'PandoraCaloCalibration_calid_sixteen_workerid_twenty'), 149814, self)

  def test_calcjobresubmittal(self):  # pylint: disable=too-many-branches
    """Test calculateJobsToBeResubmitted function."""
    workermappings = [{}, {}, {}, {}]
    for i in range(0, 4):
      if i == 0:  # 100 jobs in total, 87 are ok - expectation: No jobs to be resubmitted
        for j in range(25, 301, 6):  # adds 46 jobs
          workermappings[i][j] = 'Running'
        assert len(workermappings[i]) == 46
        for j in range(28, 274, 6):  # adds 41 jobs
          workermappings[i][j] = 'Finished'
        assert len(workermappings[i]) == 87
        for j in range(1, 25, 2):  # adds 12 jobs
          workermappings[i][j] = 'Killed'
        workermappings[i][2] = 'Failed'
        assert len(workermappings[i]) == 100
      elif i == 1:  # 20 jobs in total, 10 failed - expectation: Resubmit those 10
        for j in range(13, 73, 6):  # adds 10 jobs
          workermappings[i][j] = 'Finished'
        assert len(workermappings[i]) == 10
        for j in range(100, 128, 4):  # adds 7 jobs
          workermappings[i][j] = 'Killed'
        assert len(workermappings[i]) == 17
        for j in range(1, 7, 2):  # adds 3 jobs
          workermappings[i][j] = 'Failed'
        assert len(workermappings[i]) == 20
      elif i == 2:  # 1200 jobs, all running - expectation: No resubmission
        for j in range(1, 1201):  # adds 1200 jobs
          workermappings[i][j] = 'Running'
        assert len(workermappings[i]) == 1200
      else:  # 35 jobs, all killed - expectation: Resubmit all
        for j in range(1, 70, 2):  # adds 35 jobs
          workermappings[i][j] = 'Killed'
        assert len(workermappings[i]) == 35
    jobStatusDict = {89214: workermappings[0], 9824: workermappings[1],
                     9135: workermappings[2], 98245: workermappings[3]}
    targetNumberDict = {89214: 100, 9824: 20, 9135: 1200, 98245: 35}
    result = self.calag._CalibrationAgent__calculateJobsToBeResubmitted(jobStatusDict, targetNumberDict)
    countResubmissions = defaultdict(int)  # defaults to 0
    for calibrationID, _ in result:
      countResubmissions[calibrationID] += 1
    assertEqualsImproved((countResubmissions[89124], countResubmissions[9824],
                          countResubmissions[9135], countResubmissions[98245]),
                         (0, 10, 0, 35), self)
    expected_resubmissions = [[], itertools.chain(range(100, 128, 4), range(1, 7, 2)),
                              [], range(1, 70, 2)]
    calibIDs = [89124, 9824, 9135, 98245]
    assert len(expected_resubmissions) == len(calibIDs)
    for i, expectation in zip(calibIDs, expected_resubmissions):
      for expected_workerid in expectation:
        assertInImproved((i, expected_workerid), result, self)

  def test_requestResubmission(self):
    """Assert nothing is thrown."""
    self.calag.requestResubmission([(13875, 137), (1735, 1938), (90452, 4981)])
    self.rpc_mock().resubmitJobs.assert_called_once_with([(13875, 137), (1735, 1938), (90452, 4981)])

  def test_requestResubmission_permanent_fail(self):
    """Calibration 1 works, Calibration 5 always fails."""
    def mock_resubmit(failedJobs):
      """Mock the resubmission method of the service."""
      if ((1, 419857) in failedJobs and len(failedJobs) == 4) or \
         (len(failedJobs) == 1 and (5, 713) in failedJobs):
        result = S_ERROR('Could not resubmit all jobs. Failed calibration/worker pairs are: [(5,713)]')
        result['failed_pairs'] = [(5, 713)]
        return result
      else:
        raise IOError('test failed. list should never be empty.')
    self.rpc_mock().resubmitJobs.side_effect = mock_resubmit
    with pytest.raises(RuntimeError) as re:
      self.calag.requestResubmission([(1, 2847), (5, 713), (1, 419857), (1, 1498)])
      assertInImproved('cannot resubmit the necessary failed jobs', re.message.lower(), self)
      assertInImproved('5,713', re.message.lower(), self)

  #  def test_requestResubmission_fail_then_success( self ):
  #    def mock_resubmit( failedJobs ):
  #      """ Mocks the resubmission method of the service """
  #      print 'failedJobs: %s' % failedJobs
  #      if ( 3, 13135 ) in failedJobs and len( failedJobs ) == 5:
  #        result = S_ERROR( 'Could not resubmit all jobs. Failed calibration/worker pairs are: [(6,39105),(2,1843)]' )
  #        result[ 'failed_pairs' ] = [ ( 6, 39105 ), ( 2, 1843 ) ]
  #        return result
  #      elif len( failedJobs ) == 2 and ( 2, 1843 ) in failedJobs and ( 6, 39105 ) in failedJobs:
  #        result = S_ERROR( 'Could not resubmit all jobs. Failed calibration/worker pairs are: [(6,39105)' )
  #        result[ 'failed_pairs' ] = [ ( 6, 39105 ) ]
  #        return result
  #      elif len( failedJobs ) == 1 and ( 6, 39105 ) in failedJobs:
  #        return S_OK()
  #      else:
  #        raise IOError( 'test failed. list should never be empty.' )
  #    self.rpc_mock().resubmitJobs.side_effect = mock_resubmit
  #    # assert nothing is thrown
  #    self.calag.requestResubmission( [ ( 2, 1843 ), ( 3, 19485 ), ( 3, 13135 ), ( 3, 1835 ), ( 6, 39105 ) ] )

  def test_fetchJobStatuses(self):
    """Test getJobsParameters function."""
    jobmon_mock = Mock(name='jobmon_mock')

    def mock_getJobs(inDict):
      if '64' in inDict['JobGroup']:
        return S_OK([417251, 12741])
      elif '65' in inDict['JobGroup']:
        return S_OK([4178, 444])
      elif '66' in inDict['JobGroup']:
        return S_OK([555])
      else:
        return S_ERROR()

    jobmon_mock().getJobs.side_effect = mock_getJobs
    jobmon_mock().getJobsParameters.return_value = S_OK({
        'some_cal_1': {'JobName': 'PandoraCaloCalibration_calid_64_workerid_1', 'Status': 'Running', 'JobID': 417251,
                       'Owner': 'ow1', 'OwnerGroup': 'owGr1', 'OwnerDN': 'dummy'},
        'some_cal_2': {'JobName': 'PandoraCaloCalibration_calid_64_workerid_2', 'Status': 'Failed', 'JobID': 12741,
                       'Owner': 'ow1', 'OwnerGroup': 'owGr1', 'OwnerDN': 'dummy'},
        'some_cal_3': {'JobName': 'PandoraCaloCalibration_calid_65_workerid_5', 'Status': 'Running', 'JobID': 4178,
                       'Owner': 'ow2', 'OwnerGroup': 'owGr2', 'OwnerDN': 'dummy'},
        'some_cal_4': {'JobName': 'PandoraCaloCalibration_calid_65_workerid_6', 'Status': 'Finished', 'JobID': 444,
                       'Owner': 'ow2', 'OwnerGroup': 'owGr2', 'OwnerDN': 'dummy'},
        'some_other_cal': {'JobName': 'PandoraCaloCalibration_calid_66_workerid_14', 'Status': 'Killed', 'JobID': 555,
                           'Owner': 'ow3', 'OwnerGroup': 'owGr3', 'OwnerDN': 'dummy'}})
    calibservice_mock = Mock(name='calibServiceMock')
    calibservice_mock().getActiveCalibrations.return_value = S_OK([64, 65, 66])
    with patch('%s.JobMonitoringClient' % MODULE_NAME, new=jobmon_mock):
      with patch('%s.Client' % MODULE_NAME, new=calibservice_mock):
        self.calag.initialize()
        status_dict = self.calag.fetchJobStatuses()
        dict1 = {64: {1: 'Running', 2: 'Failed'},
                 65: {5: 'Running', 6: 'Finished'},
                 66: {14: 'Killed'}}
        dict2 = {64: {417251: 'Running', 12741: 'Failed'},
                 65: {4178: 'Running', 444: 'Finished'},
                 66: {555: 'Killed'}}
        dict3 = {64: {'Owner': 'ow1', 'OwnerGroup': 'owGr1', 'OwnerDN': 'dummy'},
                 65: {'Owner': 'ow2', 'OwnerGroup': 'owGr2', 'OwnerDN': 'dummy'},
                 66: {'Owner': 'ow3', 'OwnerGroup': 'owGr3', 'OwnerDN': 'dummy'}}
        assertEqualsImproved(status_dict, S_OK({'jobStatusVsWorkerId': dict1,
                                                'jobStatusVsJobId': dict2, 'calibrationOwnership': dict3}), self)
    jobmon_mock().getJobs.assert_called_with({'JobGroup': 'PandoraCaloCalibration_calid_66'})
    jobmon_mock().getJobsParameters.assert_called_once_with([417251, 12741, 4178, 444, 555], [
        'JobName', 'Status', 'JobID', 'Owner', 'OwnerGroup', 'OwnerDN'])



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Helper methods to create proper tarballs."""

from __future__ import print_function
from __future__ import absolute_import
import os
import subprocess
import re

from DIRAC import S_OK, gLogger

RSYNCBASE = "rsync --exclude '.svn'"


def getStatusOutput(*args, **kwargs):
  """Call command and return tuple of (status, out, err)."""
  subPipe = subprocess.Popen(*args, **kwargs)
  stdout, stderr = subPipe.communicate()
  return subPipe.returncode, stdout, stderr

def killRPath(folder):
  """remove rpath from all libraries in folder and below."""
  for root, _dirs, files in os.walk(folder, followlinks=True):
    for fil in files:
      if fil.count(".so"):
        getStatusOutput("chrpath -d %s " % os.path.join(root, fil))


def getFiles(folder, ext):
  """return a list of files with ext in the folder."""
  libraries = set()
  for root, _dirs, files in os.walk(folder, followlinks=True):
    for fil in files:
      if fil.count(ext):
        fullPath = os.path.join(root, fil)
        libraries.add(fullPath)
  return libraries


def copyLibraries(files, targetFolder, ):
  """rsync all the files to the targetFolder."""
  print("Copying files to", targetFolder)
  listOfFiles = " ".join(files)
  status, copyOut, copyErr = getStatusOutput(RSYNCBASE + " -avzL  %s %s " % (listOfFiles, targetFolder))
  if status != 0:
    print("output", copyOut)
    print("error", copyErr)
    raise RuntimeError("Error during rsync")


def resolveLinks(targetFolder):
  """if library is there twice make a link from one to the other."""
  cwd = os.getcwd()
  os.chdir(targetFolder)
  files = getFiles(targetFolder, ".so")
  print(files)
  files = [os.path.basename(fil) for fil in files]
  print("files", files)
  for lib in files:
    matchingLib = next((x for x in files if lib + "." in x), None)
    if matchingLib:
      print("going to link", lib, "with", matchingLib)
      os.remove(lib)
      os.symlink(os.path.basename(matchingLib), os.path.basename(lib))
  os.chdir(cwd)


def getLibraryPath(basePath):
  """return the path to the libraryfolder."""
  return os.path.join(basePath, "lib")


def copyFolder(basePath, targetFolder):
  """copy folder basePath to targetFolder."""
  commandString = RSYNCBASE + " -avzL  %s %s " % (basePath, targetFolder)
  print(commandString)
  try:
    os.makedirs(targetFolder)
  except OSError:
    pass
  status, copyOut, copyErr = getStatusOutput(commandString)
  if status != 0:
    print("output", copyOut)
    print("error", copyErr)
    raise RuntimeError("Error during rsync")


def getPythonStuff(basePath, targetFolder):
  """copy the python stuff from basePath to targetFolder."""
  copyFolder(basePath, targetFolder)


def removeSystemLibraries(folder):
  """remove the system libraries from the folder.

  .. todo:: get these libraries from ILCDIRAC

  |  for file in libc.so* libc-2.5.so* libm.so* libpthread.so* libdl.so* libstdc++.so* libgcc_s.so.1*; do
  |    rm $LIBFOLDER/$file 2> /dev/null
  |  done
  """
  systemLibraires = ['libc.so', 'libc-2.5.so', 'libm.so', 'libpthread.so', 'libdl.so',
                     'libstdc++.so', 'libgcc_s.so.1']

  for root, _dirs, files in os.walk(folder):
    for fil in files:
      if any(lib in fil for lib in systemLibraires):
        fullPath = os.path.join(root, fil)
        print("Removing:", fullPath)
        try:
          os.remove(fullPath)
        except OSError:
          print("Error to remove", fullPath)


def getDependentLibraries(library):
  """get all shared objects the library depends on string1=$(ldd $programpath | grep "=>" | sed 's/.*=>//g' | sed "s/(.*)//g")"""
  libraries = set()
  _status, lddOutput, _lddErr = getStatusOutput(" ".join(["ldd", library]))
  for line in lddOutput.splitlines():
    if "not found" in line:
      raise RuntimeError("Environment is not complete!: %s" % line)
    match = re.match(r'\t.*=> (.*) \(0x', line)
    if match:
      libraries.add(match.group(1))
  return libraries


def getGeant4DataFolders(variable, targetFolder):
  path = os.environ[variable]
  copyFolder(path, targetFolder)


def getRootStuff(rootsys, targetFolder):
  """copy the root stuff we need."""
  print("Copying Root")
  status, copyOut, copyErr = getStatusOutput(
      RSYNCBASE + " -av %(rootsys)s/lib %(rootsys)s/etc %(rootsys)s/bin %(rootsys)s/cint  %(targetFolder)s" %
      dict(rootsys=rootsys, targetFolder=targetFolder))
  if status != 0:
    print("output", copyOut)
    print("error", copyErr)
    raise RuntimeError("Error during rsync")

  libraries = getFiles(targetFolder + "/lib", ".so")
  allLibs = set()
  for lib in libraries:
    allLibs.update(getDependentLibraries(lib))
  copyLibraries(allLibs, targetFolder + "/lib")


def insertCSSection(csAPI, path, pardict):
  """insert a section and values (or subsections) into the CS.

  :param str path: full path of the new section
  :param str pardict: dictionary of key values in the new section, values can also be dictionaries
  :return: S_OK(), S_ERROR()
  """

  for key, value in pardict.items():
    newSectionPath = os.path.join(path, key)
    gLogger.debug("Adding to cs %s : %s " % (newSectionPath, value))
    csAPI.createSection(path)
    if isinstance(value, dict):
      res = insertCSSection(csAPI, newSectionPath, value)
    else:
      res = csAPI.setOption(newSectionPath, value)

    if not res['OK']:
      return res
    else:
      gLogger.notice("Added to CS: %s " % res['Value'])

  return S_OK("Added all things to cs")



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Create a tarball, upload and register a version for the ddsim application.

.. warning ::

  One should use CVMFS and `dirac-ilc-add-cvmfs-software` instead.

The script takes ``ddsim`` from the current environment.
"""

from __future__ import print_function
from __future__ import absolute_import
import sys
import os
import tarfile

try:
  import hashlib as md5
except ImportError:
  import md5

from pprint import pprint


from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import gLogger, S_OK


class DDSimTarMaker(object):
  """create a tarball of the DDSim release."""

  def __init__(self):
    from DIRAC.ConfigurationSystem.Client.CSAPI import CSAPI

    self.detmodels = {}
    self.lcgeo_env = "lcgeo_DIR"
    self.ddhep_env = "DD4HEP"
    self.softSec = "/Operations/Defaults/AvailableTarBalls"
    self.version = ''
    self.platform = 'x86_64-slc5-gcc43-opt'
    self.comment = ""
    self.name = "ddsim"
    self.csapi = CSAPI()
    self.tarBallName = None
    self.md5sum = None

  def copyDetectorModels(self, basePath, folder, targetFolder):
    """copy the compact folders to the targetFolder."""
    from ILCDIRAC.ILCTransformationSystem.Utilities.ReleaseHelper import copyFolder
    for root, dirs, _files in os.walk(os.path.join(basePath, folder)):
      for direct in dirs:
        if root.endswith("compact"):
          # the main xml file must have the same name as the folder, ILD and CLIC follow this convention already
          xmlPath = os.path.join(root, direct, direct + ".xml")
          if os.path.exists(xmlPath):
            self.detmodels[direct] = "detectors/" + direct + "/" + direct + ".xml"
            copyFolder(os.path.join(root, direct), targetFolder)

  def createTarBall(self, folder):
    """create a tar ball from the folder tar zcf $TARBALLNAME $LIBFOLDER/*"""
    # Create the Tarball
    if os.path.exists(self.tarBallName):
      os.remove(self.tarBallName)
    gLogger.notice("Creating Tarball...")
    myappTar = tarfile.open(self.tarBallName, "w:gz")
    myappTar.add(folder, self.tarBallName[:-4])
    myappTar.close()

    self.md5sum = md5.md5(open(self.tarBallName, 'r').read()).hexdigest()

    gLogger.notice("...Done")
    return S_OK("Created Tarball")

  def parseArgs(self):
    """parse the command line arguments."""
    if len(sys.argv) != 3:
      raise RuntimeError("Wrong number of arguments in call: '%s'" % " ".join(sys.argv))
    self.name = sys.argv[1]
    self.version = sys.argv[2]
    self.tarBallName = "%s%s.tgz" % (self.name, self.version)

  def checkEnvironment(self):
    """check if dd4hep and lcgeo are in the environment."""
    for var in [self.ddhep_env, self.lcgeo_env, 'ROOTSYS']:
      if var not in os.environ:
        raise RuntimeError("%s is not set" % var)
    return os.environ[self.ddhep_env], os.environ[self.lcgeo_env], os.environ['ROOTSYS']

  def createCSEntry(self):
    """add the entries for this version into the Configuration System.

    .. code::

      <version>
              {
                TarBall = ddsim<version>.tgz
                AdditionalEnvVar
                {
                  ROOTSYS = /cvmfs/ilc.desy.de/sw/x86_64_gcc44_sl6/root/5.34.30
                  G4INSTALL = /cvmfs/ilc.desy.de/sw/x86_64_gcc44_sl6/geant4/10.01
                  G4DATA = /cvmfs/ilc.desy.de/sw/x86_64_gcc44_sl6/geant4/10.01/share/Geant4-10.1.0/data
                }
                Overwrite = True
              }
      Operations/DDSimDetectorModels/<Version>
                {
                  CLIC_o2_v03 = detectors/CLIC_o2_v03/CLIC_o2_v03.xml
                  ...
                }
    """
    from ILCDIRAC.Core.Utilities.CheckAndGetProdProxy import checkOrGetGroupProxy
    from ILCDIRAC.ILCTransformationSystem.Utilities.ReleaseHelper import insertCSSection
    # FIXME: Get root and geant4 location from environment, make sure it is cvmfs
    csParameter = {"TarBall": self.tarBallName,
                    "AdditionalEnvVar": {
                        "ROOTSYS": os.path.realpath(os.environ.get("ROOTSYS")),
                        "G4INSTALL": os.path.realpath(os.environ.get("G4INSTALL")),
                        },
                   "Md5Sum": self.md5sum,
        }

    g4datavariables = ["G4RADIOACTIVEDATA",
                        "G4NEUTRONHPDATA",
                        "G4LEDATA",
                        "G4LEVELGAMMADATA",
                        "G4RADIOACTIVEDATA",
                        "G4NEUTRONXSDATA",
                        "G4PIIDATA",
                        "G4REALSURFACEDATA",
                        "G4SAIDXSDATA",
                        "G4ABLADATA",
                        "G4ENSDFSTATEDATA",
                      ]
    for g4data in g4datavariables:
      if os.environ.get(g4data):
        csParameter["AdditionalEnvVar"][g4data] = os.path.realpath(os.environ.get(g4data))

    pars = dict(platform=self.platform,
                 name="ddsim",
                 version=self.version
               )

    csPath = os.path.join(self.softSec, "%(platform)s/%(name)s/%(version)s/" % pars)
    pprint(csParameter)
    result = insertCSSection(self.csapi, csPath, csParameter)

    csPathModels = "Operations/Defaults/DDSimDetectorModels"
    csModels = {self.version: self.detmodels}
    pprint(csModels)

    result = insertCSSection(self.csapi, csPathModels, csModels)

    if self.csapi is not None:
      resProxy = checkOrGetGroupProxy("diracAdmin")
      if not resProxy['OK']:
        gLogger.error("Failed to get AdminProxy", resProxy['Message'])
        raise RuntimeError("Failed to get diracAdminProxy")
      self.csapi.commit()

    if not result['OK']:
      gLogger.error("Failed to create CS Section", result['Message'])
      raise RuntimeError("Failed to create CS Section")

  def createDDSimTarBall(self):
    """do everything to create the DDSim tarball."""

    from ILCDIRAC.ILCTransformationSystem.Utilities.ReleaseHelper import copyFolder, getLibraryPath, getFiles, getDependentLibraries, copyLibraries, getPythonStuff, killRPath, resolveLinks, removeSystemLibraries

    self.parseArgs()
    ddBase, lcgeoBase, _rootsys = self.checkEnvironment()

    realTargetFolder = os.path.join(os.getcwd(), self.name + self.version)
    targetFolder = os.path.join(os.getcwd(), "temp", self.name + self.version)
    for folder in (targetFolder, targetFolder + "/lib"):
      try:
        os.makedirs(folder)
      except OSError:
        pass

    libraries = set()
    rootmaps = set()

    dd4hepLibPath = getLibraryPath(ddBase)
    lcgeoPath = getLibraryPath(lcgeoBase)

    # FIXME: Automatically pick up folders with /compact/ in them
    self.copyDetectorModels(lcgeoBase, "CLIC", targetFolder + "/detectors")
    self.copyDetectorModels(lcgeoBase, "ILD", targetFolder + "/detectors")
    self.copyDetectorModels(lcgeoBase, "SiD", targetFolder + "/detectors")

    copyFolder(ddBase + "/DDDetectors/compact", realTargetFolder.rstrip("/") + "/DDDetectors")

    copyFolder(ddBase + "/include", realTargetFolder.rstrip("/") + "")

    libraries.update(getFiles(dd4hepLibPath, ".so"))
    libraries.update(getFiles(lcgeoPath, ".so"))

    rootmaps.update(getFiles(dd4hepLibPath, ".rootmap"))
    rootmaps.update(getFiles(lcgeoPath, ".rootmap"))

    rootmaps.update(getFiles(dd4hepLibPath, ".pcm"))
    rootmaps.update(getFiles(lcgeoPath, ".pcm"))

    rootmaps.update(getFiles(dd4hepLibPath, ".components"))
    rootmaps.update(getFiles(lcgeoPath, ".components"))

    pprint(libraries)
    pprint(rootmaps)

    allLibs = set()
    for lib in libraries:
      allLibs.update(getDependentLibraries(lib))
    # remote root and geant4 libraries, we pick them up from
    allLibs = set([lib for lib in allLibs if not ("/geant4/" in lib.lower() or "/root/" in lib.lower())])

    print(allLibs)

    copyLibraries(libraries, targetFolder + "/lib")
    copyLibraries(allLibs, targetFolder + "/lib")
    copyLibraries(rootmaps, targetFolder + "/lib")

    getPythonStuff(ddBase + "/python", targetFolder + "/lib/")
    getPythonStuff(lcgeoBase + "/lib/python", targetFolder + "/lib/")
    getPythonStuff(lcgeoBase + "/bin/ddsim", targetFolder + "/bin/")

    # Should get this from CVMFS
    #getRootStuff( rootsys, targetFolder+"/ROOT" )

    copyFolder(targetFolder + "/", realTargetFolder.rstrip("/"))

    killRPath(realTargetFolder)
    resolveLinks(realTargetFolder + "/lib")
    removeSystemLibraries(realTargetFolder + "/lib")
    #removeSystemLibraries( realTargetFolder+"/ROOT/lib" )
    print(self.detmodels)

    self.createTarBall(realTargetFolder)

    self.createCSEntry()


@DIRACScript()
def main():
  Script.parseCommandLine(ignoreErrors=False)
  print("Creating Tarball for DDSim")
  try:
    DDSIMMAKER = DDSimTarMaker()
    DDSIMMAKER.createDDSimTarBall()
  except RuntimeError as e:
    print("ERROR during tarball creation: %s " % e)
    exit(1)
  exit(0)


if __name__ == "__main__":
  main()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""UploadLogFile module is used to upload the files present in the working directory.

:author: S. Poss
:since: Sep 01, 2010
"""
from __future__ import absolute_import
import os
import shutil
import glob
import random
import tarfile

from DIRAC.DataManagementSystem.Client.FailoverTransfer import FailoverTransfer
from DIRAC.RequestManagementSystem.Client.Operation import Operation
from DIRAC.RequestManagementSystem.Client.File import File
from DIRAC.Resources.Storage.StorageElement import StorageElementItem as StorageElement
from DIRAC import S_OK, S_ERROR, gLogger, gConfig

from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Core.Utilities.ProductionData import getLogPath, getExperimentFromPath
import six

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class UploadLogFile(ModuleBase):
  """Handle log file uploads in the production jobs."""
  #############################################################################

  def __init__(self):
    """Module initialization."""
    super(UploadLogFile, self).__init__()
    self.version = __RCSID__
    self.productionID = None
    self.jobID = None
    self.workflow_commons = None
    self.logFilePath = ""
    self.logLFNPath = ""
    self.logdir = ''
    self.logSE = StorageElement(self.ops.getValue('/LogStorage/LogSE', 'LogSE'))
    self.root = gConfig.getValue('/LocalSite/Root', os.getcwd())
    self.logSizeLimit = self.ops.getValue('/LogFiles/SizeLimit', 20 * 1024 * 1024)
    self.logExtensions = []
    self.failoverSEs = gConfig.getValue('/Resources/StorageElementGroups/Tier1-Failover', [])
    self.experiment = 'CLIC'
    self.enable = True
    self.failoverTest = False  # flag to put log files to failover by default
    self.jobID = ''

######################################################################
  def applicationSpecificInputs(self):
    """resolves the module parameters."""

    self.enable = self.step_commons.get('Enable', self.enable)
    if not isinstance(self.enable, bool):
      LOG.warn('Enable flag set to non-boolean value %s, setting to False' % self.enable)
      self.enable = False

    if not isinstance(self.failoverTest, bool):
      LOG.warn('Test failover flag set to non-boolean value %s, setting to False' % self.failoverTest)
      self.failoverTest = False

    self.jobID = os.environ.get('JOBID', self.jobID)
    if self.jobID != 0:
      LOG.verbose('Found WMS JobID = %s' % self.jobID)
    else:
      LOG.info('No WMS JobID found, disabling module via control flag')
      self.enable = False

    self.logFilePath = self.workflow_commons.get('LogFilePath', self.logFilePath)
    self.logLFNPath = self.workflow_commons.get('LogTargetPath', self.logLFNPath)
    if not (self.logFilePath and self.logLFNPath):
      LOG.info('LogFilePath parameter not found, creating on the fly')
      result = getLogPath(self.workflow_commons)
      if not result['OK']:
        LOG.error('Could not create LogFilePath', result['Message'])
        return result
      self.logFilePath = result['Value']['LogFilePath'][0]
      self.logLFNPath = result['Value']['LogTargetPath'][0]

    if not isinstance(self.logFilePath, six.string_types):
      self.logFilePath = self.logFilePath[0]
    if not isinstance(self.logLFNPath, six.string_types):
      self.logLFNPath = self.logLFNPath[0]

    self.experiment = getExperimentFromPath(LOG, self.logFilePath, self.experiment)

    return S_OK('Parameters resolved')

######################################################################
  def execute(self):
    """Main execution method."""
    LOG.info('Initializing %s' % self.version)
    # Add global reporting tool
    res = self.resolveInputVariables()
    if not res['OK']:
      LOG.error("Failed to resolve input parameters:", res['Message'])

    self.logWorkingDirectory()
    LOG.info('Job root is found to be %s' % (self.root))
    LOG.info('PRODUCTION_ID = %s, JOB_ID = %s ' % (self.productionID, self.jobID))
    self.logdir = os.path.realpath('./job/log/%s/%s' % (self.productionID, self.jobID))
    LOG.info('Selected log files will be temporarily stored in %s' % self.logdir)

    res = self.finalize()
    return res

  #############################################################################
  def finalize(self):
    """finalize method performs final operations after all the job steps were executed.

    Only production jobs are treated.
    """

    LOG.verbose('Starting UploadLogFile finalize')
    LOG.debug("LogFilePath: %s" % self.logFilePath)
    LOG.debug("LogLFNPath:  %s" % self.logLFNPath)

    ##########################################
    # First determine the files which should be saved
    LOG.info('Determining the files to be saved in the logs.')
    resRelevantFiles = self._determineRelevantFiles()
    if not resRelevantFiles['OK']:
      LOG.error('Completely failed to select relevant log files.', resRelevantFiles['Message'])
      return S_OK()  # because if the logs are lost, it's not the end of the world.

    selectedFiles = resRelevantFiles['Value']
    if not selectedFiles:
      LOG.info("No log files selected")
      return S_OK()

    LOG.info('The following %s files were selected to be saved:\n%s' % (len(selectedFiles),
                                                                             "\n".join(selectedFiles)))

    #########################################
    # Create a temporary directory containing these files
    LOG.info('Populating a temporary directory for selected files.')
    resLogDir = self._populateLogDirectory(selectedFiles)
    if not resLogDir['OK']:
      LOG.error('Completely failed to populate temporary log file directory.', resLogDir['Message'])
      self.setApplicationStatus('Failed To Populate Log Dir')
      return S_OK('Empty Log Directory')  # because if the logs are lost, it's not the end of the world.
    LOG.info('%s populated with log files.' % self.logdir)

    if not self.enable:
      LOG.info('Module is disabled by control flag')
      return S_OK('Module is disabled by control flag')

    #########################################
    # Make sure all the files in the log directory have the correct permissions
    result = self._setLogFilePermissions(self.logdir)
    if not result['OK']:
      LOG.error('Could not set permissions of log files to 0755 with message:\n%s' % (result['Message']))

    #########################################
    # Tar all the files
    resTarLogs = self._tarTheLogFiles()
    if not resTarLogs['OK']:
      return S_OK()  # because if the logs are lost, it's not the end of the world.
    tarFileName = resTarLogs['Value']['fileName']

    #########################################
    # Attempt to upload log tar ball to the LogSE
    LOG.info('Transferring log tarball to the %s' % self.logSE.name)
    resTransfer = S_ERROR("Skipping log upload, because failoverTest")
    tarFileLFN = '%s/%s' % (self.logFilePath, tarFileName)
    tarFileLocal = os.path.realpath(os.path.join(self.logdir, tarFileName))
    if not self.failoverTest:
      LOG.info('PutFile %s %s %s' % (tarFileLFN, tarFileLocal, self.logSE.name))
      resTransfer = self.logSE.putFile({tarFileLFN: tarFileLocal})
      LOG.debug("putFile result: %s" % resTransfer)
      if not resTransfer['OK']:
        LOG.error('Completely failed to upload log files to %s, will attempt upload to failover SE' % self.logSE.name,
                       resTransfer['Message'])
      elif resTransfer['OK'] and len(resTransfer['Value']['Failed']) == 0:
        LOG.info('Successfully upload log tarball to %s' % self.logSE.name)
        self.setJobParameter('Log LFN', tarFileLFN)
        LOG.info('Logs for this job may be retrieved with dirac-ilc-get-prod-log -F %s' % tarFileLFN)
        return S_OK()
      else:
        LOG.error('Completely failed to upload log files to %s, will attempt upload to failover SE' % self.logSE.name,
                       resTransfer['Value'])

    #########################################
    # Recover the logs to a failover storage element

    # if res['Value'][0]: #i.e. non-zero status
    #  LOG.error('Failed to create tar file from directory','%s %s' % (self.logdir,res['Value']))
    #  self.setApplicationStatus('Failed To Create Log Tar Dir')
    #  return S_OK()#because if the logs are lost, it's not the end of the world.

    ############################################################
    # Instantiate the failover transfer client with the global request object
    resFailover = self._tryFailoverTransfer(tarFileName, self.logdir)
    if not resFailover['Value']:
      return resFailover  # log files lost, but who cares...

    self.workflow_commons['Request'] = resFailover['Value']['Request']
    uploadedSE = resFailover['Value']['uploadedSE']
    res = self._createLogUploadRequest(self.logSE.name, self.logLFNPath, uploadedSE)
    if not res['OK']:
      LOG.error('Failed to create failover request', res['Message'])
      self.setApplicationStatus('Failed To Upload Logs To Failover')
    else:
      LOG.info('Successfully created failover request')
      LOG.info("Request %s" % self._getRequestContainer())
    return S_OK()

  #############################################################################
  def _determineRelevantFiles(self):
    """The files which are below a configurable size will be stored in the logs.

    This will typically pick up everything in the working directory minus the output data files.
    """
    logFileExtensions = ['*.txt', '*.log', '*.out', '*.output', '*.xml', '*.sh', '*.info', '*.err', '*.root']
    self.logExtensions = self.ops.getValue('/LogFiles/%s/Extensions' % self.experiment, [])

    if self.logExtensions:
      LOG.info('Using list of log extensions from CS:\n%s' % (', '.join(self.logExtensions)))
      logFileExtensions = self.logExtensions
    else:
      LOG.info('Using default list of log extensions:\n%s' % (', '.join(logFileExtensions)))

    candidateFiles = []
    for ext in logFileExtensions:
      LOG.debug('Looking at log file wildcard: %s' % ext)
      globList = glob.glob(ext)
      for check in globList:
        if os.path.isfile(check):
          LOG.debug('Found locally existing log file: %s' % check)
          candidateFiles.append(check)

    selectedFiles = []
    try:
      for candidate in candidateFiles:
        fileSize = os.stat(candidate)[6]
        if fileSize < self.logSizeLimit:
          selectedFiles.append(candidate)
        else:
          LOG.error('Log file found to be greater than maximum of %s bytes' % self.logSizeLimit, candidate)
      return S_OK(selectedFiles)
    except OSError as x:
      LOG.exception('Exception while determining files to save.', '', str(x))
      return S_ERROR('Could not determine log files')

  #############################################################################
  def _populateLogDirectory(self, selectedFiles):
    """A temporary directory is created for all the selected files.

    These files are then copied into this directory before being uploaded
    """
    # Create the temporary directory
    try:
      if not os.path.exists(self.logdir):
        os.makedirs(self.logdir)
    except EnvironmentError as x:
      LOG.exception('PopulateLogDir: Exception while trying to create directory.', self.logdir, str(x))
      return S_ERROR()
    # Set proper permissions
    LOG.info('PopulateLogDir: Changing log directory %s permissions to 0755' % self.logdir)
    try:
      os.chmod(self.logdir, 0o755)
    except EnvironmentError as x:
      LOG.error('PopulateLogDir: Could not set logdir permissions to 0755:', '%s (%s)' % (self.logdir, str(x)))
    # Populate the temporary directory
    try:
      for myfile in selectedFiles:
        destinationFile = '%s/%s' % (self.logdir, os.path.basename(myfile))
        shutil.copy(myfile, destinationFile)
    except EnvironmentError as x:
      LOG.exception('PopulateLogDir: Exception while trying to copy file.', myfile, str(x))
      LOG.info('PopulateLogDir: File %s will be skipped and can be considered lost.' % myfile)

    # Now verify the contents of our target log dir
    successfulFiles = os.listdir(self.logdir)
    LOG.info("PopulateLogDir: successfulFiles %s" % successfulFiles)
    if successfulFiles:
      LOG.info('PopulateLogDir: Prepared %s files in the temporary directory.' % len(successfulFiles))
      return S_OK()
    LOG.info('PopulateLogDir: Failed to copy any files to the target directory.')
    return S_ERROR()

  #############################################################################
  def _createLogUploadRequest(self, targetSE, logFileLFN, uploadedSE):
    """Set a request to upload job log files from the output sandbox Changed to be similar to LHCb createLogUploadRequest using LHCb LogUpload Request and Removal Request."""
    LOG.info('Setting log upload request for %s at %s' % (targetSE, logFileLFN))
    request = self._getRequestContainer()

    logUpload = Operation()
    logUpload.Type = "LogUpload"
    logUpload.TargetSE = targetSE

    upFile = File()
    upFile.LFN = logFileLFN
    logUpload.addFile(upFile)

    logRemoval = Operation()
    logRemoval.Type = 'RemoveFile'
    logRemoval.TargetSE = uploadedSE
    logRemoval.addFile(upFile)

    request.addOperation(logUpload)
    request.addOperation(logRemoval)

    self.workflow_commons['Request'] = request

    return S_OK()

  #############################################################################
  def _setLogFilePermissions(self, logDir):
    """Sets the permissions of all the files in the log directory to ensure they are readable."""
    try:
      for toChange in os.listdir(logDir):
        if not os.path.islink('%s/%s' % (logDir, toChange)):
          LOG.debug('Changing permissions of %s/%s to 0755' % (logDir, toChange))
          os.chmod('%s/%s' % (logDir, toChange), 0o755)
    except OSError as x:
      LOG.error('Problem changing shared area permissions', str(x))
      return S_ERROR(x)

    return S_OK()

  ################################################################################
  def _tryFailoverTransfer(self, tarFileName, tarFileDir):
    """tries to upload the log tarBall to the failoverSE and creates moving request."""
    failoverTransfer = FailoverTransfer(self._getRequestContainer())
    # determine the experiment
    self.failoverSEs = self.ops.getValue("Production/%s/FailOverSE" % self.experiment, self.failoverSEs)
    catalogs = self.ops.getValue('Production/%s/Catalogs' % self.experiment, ['FileCatalog', 'LcgFileCatalog'])

    random.shuffle(self.failoverSEs)
    LOG.info("Attempting to store file %s to the following SE(s):\n%s" % (tarFileName,
                                                                               ', '.join(self.failoverSEs)))
    result = failoverTransfer.transferAndRegisterFile(tarFileName, '%s/%s' % (tarFileDir, tarFileName), self.logLFNPath,
                                                      self.failoverSEs, fileMetaDict={"GUID": None},
                                                      fileCatalog=catalogs)
    if not result['OK']:
      LOG.error('Failed to upload logs to all destinations')
      self.setApplicationStatus('Failed To Upload Logs')
      return S_OK()  # because if the logs are lost, it's not the end of the world.

    # Now after all operations, return potentially modified request object
    return S_OK({'Request': failoverTransfer.request, 'uploadedSE': result['Value']['uploadedSE']})

  def _tarTheLogFiles(self):
    """returns S_OK/S_ERROR and puts all the relevantFiles into a tarball."""
    self.logLFNPath = '%s.gz' % self.logLFNPath
    tarFileName = os.path.basename(self.logLFNPath)

    LOG.debug("Log Directory: %s" % self.logdir)
    LOG.debug("Log LFNPath:   %s" % self.logLFNPath)
    LOG.debug("TarFileName:   %s" % tarFileName)
    start = os.getcwd()
    os.chdir(self.logdir)
    logTarFiles = os.listdir(self.logdir)
    #comm = 'tar czvf %s %s' % (tarFileName,string.join(logTarFiles,' '))
    tfile = tarfile.open(tarFileName, "w:gz")
    for item in logTarFiles:
      LOG.info("Adding file %s to tarfile %s" % (item, tarFileName))
      tfile.add(item)
    tfile.close()
    resExists = S_OK() if os.path.exists(tarFileName) else S_ERROR("File was not created")
    os.chdir(start)
    LOG.debug("TarFileName: %s" % (tarFileName,))
    if resExists['OK']:
      return S_OK(dict(fileName=tarFileName))
    else:
      LOG.error('Failed to create tar file from directory', '%s %s' % (self.logdir, resExists['Message']))
      self.setApplicationStatus('Failed To Create Log Tar Dir')
      return S_ERROR()

#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Base class for ILC workflow modules.

Stolen by S. Poss from LHCbSystem.Workflow.Modules

:since: Feb 02, 2010
:author: S. Poss
:author: S. Paterson
"""

from __future__ import print_function
from __future__ import absolute_import
import os
import re
import shutil
import string
import sys
import six.moves.urllib.request
import six.moves.urllib.parse
import six.moves.urllib.error
from collections import defaultdict
from pprint import pformat
from random import choice

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Security.ProxyInfo import getProxyInfoAsString
from DIRAC.Core.Utilities.Adler import fileAdler
from DIRAC.Core.Utilities.Subprocess import shellCall
from DIRAC.TransformationSystem.Client.FileReport import FileReport
from DIRAC.WorkloadManagementSystem.Client.JobReport import JobReport
from DIRAC.Core.Utilities.File import makeGuid
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
from DIRAC.RequestManagementSystem.Client.Request import Request
from DIRAC.RequestManagementSystem.private.RequestValidator import RequestValidator
from DIRAC.RequestManagementSystem.Client.Operation import Operation
from DIRAC.RequestManagementSystem.Client.File import File

from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getSoftwareFolder, checkCVMFS
from ILCDIRAC.Core.Utilities.FindSteeringFileDir import getSteeringFileDir
from ILCDIRAC.Core.Utilities.InputFilesUtilities import getNumberOfEvents
import six
from six.moves import range

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


def generateRandomString(length=8, chars=string.ascii_letters + string.digits):
  """Return random string of 8 chars, used by :mod:`~ILCDIRAC.Workflow.Modules.PythiaAnalysis` and :mod:`~ILCDIRAC.Workflow.Modules.MokkaAnalysis`"""
  return ''.join([choice(chars) for _ in range(length)])


class ModuleBase(object):
  """Base class of the ILCDIRAC modules.

  Several common utilities are defined here.
  In particular, all sub classes should call the :func:`resolveInputVariables` method, and implement
  the :func:`applicationSpecificInputs`.
  """
  #############################################################################

  def __init__(self):
    """Initialization of module base."""
    super(ModuleBase, self).__init__()
    # FIXME: do we need to do this for every module?
    result = getProxyInfoAsString()
    if not result['OK']:
      LOG.error('Could not obtain proxy information in module environment with message:\n', result['Message'])
    else:
      LOG.info('Payload proxy information:\n', result['Value'])

    self.ops = Operations()

    self.platform = ''
    self.applicationLog = ''
    self.applicationVersion = ''
    self.applicationName = ''
    self.InputData = []  # Will have to become empty list
    self.SteeringFile = ''
    self.energy = 0
    self.NumberOfEvents = 0
    self.WorkflowStartFrom = 0
    self.result = S_ERROR()
    self.InputFile = []
    self.ignoremissingInput = False
    self.OutputFile = ''
    self.jobType = ''
    self.stdError = ''
    self.debug = False
    self.extraCLIarguments = ""
    self.jobID = os.environ.get("JOBID", 0)
    self.eventstring = ['']
    self.excludeAllButEventString = False
    self.ignoreapperrors = False
    self.inputdataMeta = {}
    #############
    # Set from workflow object
    self.workflow_commons = {}
    self.step_commons = {}
    self.workflowStatus = S_OK()
    self.stepStatus = S_OK()
    self.isProdJob = False
    self.productionID = 0
    self.prod_job_id = 0
    self.jobReport = None
    # Make sure this is set everywhere
    os.environ['OMP_NUM_THREADS'] = '1'

  #############################################################################
  def setApplicationStatus(self, status, sendFlag=True):
    """Wraps around setJobApplicationStatus of state update client."""
    if not self.jobID:
      return S_OK('JobID not defined')  # e.g. running locally prior to submission

    LOG.verbose('setJobApplicationStatus(%s,%s)' % (self.jobID, status))

    self.jobReport = self.workflow_commons.get('JobReport', self.jobReport)

    if not self.jobReport:
      return S_OK('No reporting tool given')
    jobStatus = self.jobReport.setApplicationStatus(status, sendFlag)
    if not jobStatus['OK']:
      LOG.warn(jobStatus['Message'])

    return jobStatus

  #############################################################################
  def sendStoredStatusInfo(self):
    """Wraps around sendStoredStatusInfo of state update client."""
    if not self.jobID:
      return S_OK('JobID not defined')  # e.g. running locally prior to submission

    self.jobReport = self.workflow_commons.get('JobReport', self.jobReport)

    if not self.jobReport:
      return S_OK('No reporting tool given')

    sendStatus = self.jobReport.sendStoredStatusInfo()
    if not sendStatus['OK']:
      LOG.error(sendStatus['Message'])

    return sendStatus

  #############################################################################
  def setJobParameter(self, name, value, sendFlag=True):
    """Wraps around setJobParameter of state update client.

    :param str name: job parameter
    :param value: value of the job parameter
    :param bool sendFlag: passed to setJobParameter
    :return: S_OK, S_ERROR
    """
    if not self.jobID:
      return S_OK('JobID not defined')  # e.g. running locally prior to submission

    LOG.verbose('setJobParameter(%s,%s,%s)' % (self.jobID, name, value))

    self.jobReport = self.workflow_commons.get('JobReport', self.jobReport)

    if not self.jobReport:
      return S_OK('No reporting tool given')
    jobParam = self.jobReport.setJobParameter(str(name), str(value), sendFlag)
    if not jobParam['OK']:
      LOG.warn(jobParam['Message'])

    return jobParam

  #############################################################################
  def sendStoredJobParameters(self):
    """Wraps around sendStoredJobParameters of state update client."""
    if not self.jobID:
      return S_OK('JobID not defined')  # e.g. running locally prior to submission

    self.jobReport = self.workflow_commons.get('JobReport', self.jobReport)

    if not self.jobReport:
      return S_OK('No reporting tool given')

    sendStatus = self.jobReport.sendStoredJobParameters()
    if not sendStatus['OK']:
      LOG.error(sendStatus['Message'])

    return sendStatus

  #############################################################################
  def setFileStatus(self, production, lfn, status):
    """Set the file status for the given production in the Production Database.

    :param int production: production ID
    :param str lfn: logical file name of the file that needs status change
    :param str status: status to set
    :return: S_OK(), S_ERROR()
    """
    LOG.verbose('setFileStatus(%s,%s,%s)' % (production, lfn, status))

    fileReport = self.workflow_commons.setdefault('FileReport', FileReport('Transformation/TransformationManager'))
    result = fileReport.setFileStatus(production, lfn, status)
    if not result['OK']:
      LOG.warn(result['Message'])

    self.workflow_commons['FileReport'] = fileReport

    return result

  def getCandidateFiles(self, outputList, outputLFNs, dummy_fileMask):
    """Returns list of candidate files to upload, check if some outputs are missing.

    :param outputList: has the following structure: [ ('outputFile':'','outputDataSE':'','outputPath':'') , (...) ]
    :param outputLFNs: list of output LFNs for the job
    :param dummy_fileMask:  UNUSED output file extensions to restrict the outputs to
    :return: S_OK with dictionary containing type, SE and LFN for files [NOT: restricted by mask]
    """
    fileInfo = {}
    for outputFile in outputList:
      if all(key in outputFile for key in ('outputFile', 'outputDataSE', 'outputPath')):
        fname = outputFile['outputFile']
        fileSE = outputFile['outputDataSE']
        filePath = outputFile['outputPath']
        fileInfo[fname] = {'path': filePath, 'workflowSE': fileSE}
      else:
        LOG.error('Ignoring malformed output data specification', str(outputFile))

    for lfn in outputLFNs:
      LOG.debug("Checking LFN: %s" % lfn)
      if os.path.basename(lfn) in fileInfo:
        fileInfo[os.path.basename(lfn)]['lfn'] = lfn
        LOG.verbose('Found LFN %s for file %s' % (lfn, os.path.basename(lfn)))
        if len(os.path.basename(lfn)) > 127:
          LOG.error('Your file name is WAAAY too long for the FileCatalog. Cannot proceed to upload. Filename:',
                    os.path.basename(lfn))
          return S_ERROR('Filename too long')
        if len(lfn) > 256 + 127:
          LOG.error('Your LFN is WAAAAY too long for the FileCatalog. Cannot proceed to upload. LFN:', lfn)
          return S_ERROR('LFN too long')

    # Check that the list of output files were produced
    for fileName, metadata in list(fileInfo.items()):
      if not os.path.exists(fileName):
        LOG.error('Output data file %s does not exist locally' % fileName)
        if not self.ignoreapperrors:
          return S_ERROR('Output Data Not Found')
        fileInfo.pop(fileName)
    # Check the list of files against the output file mask (if it exists)
    #candidateFiles = {}
    # if fileMask:
      # nothing to do yet, as FileMask is not used
      # for fileName,metadata in fileInfo.items():
      #  if metadata['type'].lower() in fileMask or fileName.split('.')[-1] in fileMask:
      #    candidateFiles[fileName]=metadata
      #  else:
      #    LOG.info('Output file %s was produced but will not be treated (outputDataFileMask is %s)' %(fileName,
      #                                                                    string.join(self.outputDataFileMask,', ')))

      # if not candidateFiles.keys():
      #  return S_OK({}) #nothing to do
    #  candidateFiles = fileInfo
    # else:
      # do not apply mask to files

    candidateFiles = fileInfo
    # Sanity check all final candidate metadata keys are present (return S_ERROR if not)
    mandatoryKeys = ['path', 'workflowSE', 'lfn']  # filedict is used for requests
    for fileName, metadata in candidateFiles.items():
      for key in mandatoryKeys:
        if key not in metadata:
          return S_ERROR('File %s has missing key %s' % (fileName, key))
    return S_OK(candidateFiles)

  #############################################################################
  def getFileMetadata(self, candidateFiles):
    """Returns the candidate file dictionary with associated metadata.

    :param dict candidateFiles: The input candidate files dictionary has the structure: {'lfn':'','path':'','workflowSE':''}
       This also assumes the files are in the current working directory.
    :return: S_OK with File Metadata, S_ERROR
    """
    # Retrieve the POOL File GUID(s) for any final output files
    LOG.info('Will search for POOL GUIDs for: %s' % (', '.join(list(candidateFiles.keys()))))
    pfnGUIDs = {}
    generated = []
    for fname in candidateFiles.keys():
      guid = makeGuid(fname)
      pfnGUIDs[fname] = guid
      generated.append(fname)
    pfnGUID = S_OK(pfnGUIDs)
    pfnGUID['generated'] = generated
    #pfnGUID = getGUID(candidateFiles.keys())
    # if not pfnGUID['OK']:
    #  LOG.error('PoolXMLFile failed to determine POOL GUID(s) for output file list, these will be generated by \
    #                   the DataManager',pfnGUID['Message'])
    #  for fileName in candidateFiles.keys():
    #    candidateFiles[fileName]['guid']=''
    # if pfnGUID['generated']:
    LOG.debug('Generated GUID(s) for the following files ', ', '.join(pfnGUID['generated']))
    # else:
    #  LOG.info('GUIDs found for all specified POOL files: %s' %(string.join(candidateFiles.keys(),', ')))

    for pfn, guid in pfnGUID['Value'].items():
      candidateFiles[pfn]['GUID'] = guid

    # Get all additional metadata about the file necessary for requests
    final = {}
    for fileName, metadata in candidateFiles.items():
      fileDict = {}
      fileDict['LFN'] = metadata['lfn']
      fileDict['Size'] = os.path.getsize(fileName)
      adlerChecksum = fileAdler(fileName)
      fileDict['Addler'] = adlerChecksum
      fileDict['ADLER32'] = adlerChecksum
      fileDict['Checksum'] = adlerChecksum
      fileDict['ChecksumType'] = "ADLER32"
      fileDict['GUID'] = metadata['GUID']
      fileDict['Status'] = "Waiting"

      final[fileName] = metadata
      final[fileName]['filedict'] = fileDict
      final[fileName]['localpath'] = '%s/%s' % (os.getcwd(), fileName)

    # Sanity check all final candidate metadata keys are present (return S_ERROR if not)
    mandatoryKeys = ['GUID', 'filedict']  # filedict is used for requests (this method adds guid and filedict)
    for fileName, metadata in final.items():
      for key in mandatoryKeys:
        if key not in metadata:
          return S_ERROR('File %s has missing %s' % (fileName, key))

    return S_OK(final)

  def _getRequestContainer(self):
    """just return the Request reporter (object)"""
    if 'Request' not in self.workflow_commons or not self.workflow_commons['Request'].RequestName:
      self.workflow_commons['Request'] = Request()
      self.workflow_commons['Request'].RequestName = 'job_%d_request.xml' % int(self.jobID)
      self.workflow_commons['Request'].JobID = int(self.jobID)
      self.workflow_commons['Request'].SourceComponent = "Job_%d" % int(self.jobID)
    return self.workflow_commons['Request']
  #############################################################################

  def _getJobReporter(self):
    """just return the job reporter (object, always defined by dirac-jobexec)"""

    if 'JobReport' not in self.workflow_commons:
      self.workflow_commons['JobReport'] = JobReport(self.jobID)
    return self.workflow_commons['JobReport']

  def resolveInputVariables(self):
    """Common utility for all sub classes, resolve the workflow parameters for the current step.

    Module parameters are resolved directly.
    """
    PLACE_HOLDER = "__PLACEHOLDER__"
    LOG.info("Calling module:", self.__class__)
    LOG.info("Workflow commons:\n", pformat(self.workflow_commons))
    LOG.info("Request:", pformat(self.workflow_commons.get('Request')))
    LOG.info("Step commons:\n", pformat(self.step_commons))

    for stepPar, stepValue in self.step_commons.items():
      stepValueOrg = stepValue
      LOG.debug('Checking %r with %r' % (stepPar, stepValue))
      if not isinstance(stepValue, str):
        continue
      stepValue = six.moves.urllib.parse.unquote(stepValue)

      if '%n' in stepValue:
        stepValue = stepValue.replace('%n', PLACE_HOLDER)
      stepValue = stepValue % self.workflow_commons
      if PLACE_HOLDER in stepValue:
        stepValue = stepValue.replace(PLACE_HOLDER, '%n')
      if stepValueOrg != stepValue:
        self.step_commons[stepPar] = stepValue
        LOG.info('Replaced in parameter %r with value %r to %r' % (stepPar, stepValueOrg, stepValue))

    self.jobReport = self._getJobReporter()

    self.prod_job_id = int(self.workflow_commons.get("JOB_ID", self.prod_job_id))
    if self.workflow_commons.get("IS_PROD", False):
      self.productionID = int(self.workflow_commons["PRODUCTION_ID"])
      self.isProdJob = True

    self.platform = self.workflow_commons.get('Platform', self.platform)

    # FIXME: Necessary for backward compatibility, can be removed eventually
    # (once all productions defined with v6r8/9 are gone)
    if 'Platform' not in self.workflow_commons and 'SystemConfig' in self.workflow_commons:
      self.platform = self.workflow_commons.get('SystemConfig')

    self.ignoreapperrors = self.workflow_commons.get('IgnoreAppError', self.ignoreapperrors)

    self.applicationName = self.step_commons.get('applicationName', self.applicationName)

    self.applicationVersion = self.step_commons.get('applicationVersion', self.applicationVersion)

    self.applicationLog = self.step_commons.get('applicationLog', self.applicationLog)

    self.extraCLIarguments = six.moves.urllib.parse.unquote(
        self.step_commons.get('ExtraCLIArguments', self.extraCLIarguments))

    self.SteeringFile = self.step_commons.get('SteeringFile', self.SteeringFile)

    self.jobType = self.workflow_commons.get('JobType', self.jobType)

    self.energy = self.workflow_commons.get('Energy', self.energy)

    if 'NbOfEvts' in self.workflow_commons and self.workflow_commons['NbOfEvts'] > 0:
      self.NumberOfEvents = self.workflow_commons['NbOfEvts']

    if 'StartFrom' in self.workflow_commons and self.workflow_commons['StartFrom'] > 0:
      self.WorkflowStartFrom = self.workflow_commons['StartFrom']

    if 'InputFile' in self.step_commons:
      # This must stay, otherwise, linking between steps is impossible: OutputFile is a string
      inputf = self.step_commons['InputFile']
      if not isinstance(inputf, list):
        if inputf:
          inputf = inputf.split(";")
        else:
          inputf = []
      self.InputFile = inputf

    self.ignoremissingInput = self.step_commons.get('ForgetInput', self.ignoremissingInput)

    if 'InputData' in self.workflow_commons:
      inputdata = self.workflow_commons['InputData']
      if not isinstance(inputdata, list):
        if inputdata:
          self.InputData = inputdata.split(";")
          self.InputData = [x.replace("LFN:", "") for x in self.InputData]
      else:
        self.InputData = [x.replace("LFN:", "") for x in inputdata]

    if 'ParametricInputData' in self.workflow_commons:
      paramdata = self.workflow_commons['ParametricInputData']
      if not isinstance(paramdata, list):
        if paramdata:
          self.InputData = [x.replace("LFN:", "") for x in paramdata.split(";")]
      else:
        # paramdata is a list, and we might need to get rid of "LFN:"
        self.InputData = [x.replace("LFN:", "") for x in paramdata]

    # only if OutputFile is not set
    if not self.OutputFile:
      self.OutputFile = self.step_commons.get("OutputFile", '')

    # Next is also a module parameter, should be already set
    self.debug = self.step_commons.get('debug', self.debug)

    if self.InputData and not self.NumberOfEvents:
      resNE = getNumberOfEvents(self.InputData)
      # NumberOfEvents == 0 does not necessarily mean things went wrong... This
      # is really almost(?)  impossible to solve, sometimes NumberOfEvents can
      # be 0 and then the correct number is already found in the steering file
      # provided to the job. Only in case of productions can we expect that we
      # always get number of events from somewhere. So we would need to check for production ID
      if not resNE['OK'] and self.NumberOfEvents == 0 and self.isProdJob:
        return S_ERROR("Failed to get NumberOfEvents from FileCatalog")
      if not resNE['OK'] and self.NumberOfEvents == 0 and not self.isProdJob:
        LOG.warn("Failed to get NumberOfEvents from FileCatalog, but this is not a production job")
      if resNE['OK']:
        eventsMeta = resNE['Value']
        self.inputdataMeta.update(eventsMeta['AdditionalMeta'])
        if eventsMeta["nbevts"]:
          if self.NumberOfEvents > eventsMeta['nbevts'] or self.NumberOfEvents == 0:
            self.NumberOfEvents = eventsMeta['nbevts']
      LOG.info("NumberOfEvents = %s" % str(self.NumberOfEvents))

    res = self.applicationSpecificInputs()
    if not res['OK']:
      return res
    return S_OK('Parameters resolved')

  def applicationSpecificInputs(self):
    """Method overwritten by sub classes.

    Called from the above.
    """
    return S_OK()

  def execute(self):
    """The execute method.

    This is called by the workflow wrapper when the module is needed
    Here we do preliminary things like resolving the application parameters and we definitely do not get a dedicated directory.
    """

    result = self.resolveInputVariables()
    if not result['OK']:
      LOG.error("Failed to resolve input variables:", result['Message'])
      return result

    # because we need to make sure this does not overwrite steering files provided  by the user
    # it must be here.
    if "SteeringFileVers" in self.step_commons:
      resSteering = self.treatSteeringFiles()
      if not resSteering['OK']:
        return resSteering

    resConf = self.treatConfigPackage()
    if not resConf['OK']:
      return resConf

    if self.SteeringFile:
      if os.path.exists(os.path.basename(self.SteeringFile)):
        LOG.verbose("Found local copy of %s" % self.SteeringFile)

    appres = self.runIt()
    if not appres["OK"]:
      LOG.error("Somehow the application did not exit properly")

    self.listDir()

    return appres

  def listDir(self):
    """List the current directories content."""
    ldir = os.listdir(os.getcwd())
    LOG.verbose("Base directory content:", "\n".join(ldir))

  def runIt(self):
    """Dummy call, needs to be overwritten by the actual applications."""
    return S_OK()

  def finalStatusReport(self, status):
    """Catch the resulting application status, and return corresponding workflow status."""
    message = '%s %s Successful' % (self.applicationName, self.applicationVersion)
    if status:
      LOG.error("==================================\n StdError:\n")
      LOG.error(self.stdError)
      message = '%s exited With Status %s' % (self.applicationName, status)
      self.setApplicationStatus(message)
      LOG.error(message)
      errorKey = "%s_%s" % (self.applicationName, self.applicationVersion)
      self.workflow_commons.setdefault('ErrorDict', defaultdict(list))[errorKey].extend([message, self.stdError])
      if not self.ignoreapperrors:
        return S_ERROR(message)
    else:
      self.setApplicationStatus('%s %s Successful' % (self.applicationName, self.applicationVersion))
    return S_OK(message)

  #############################################################################

  def generateFailoverFile(self):
    """Retrieve the accumulated reporting request, and produce a JSON file that is consumed by the JobWrapper."""
    request = self._getRequestContainer()
    LOG.notice("Request Before: %s" % pformat(request))
    reportRequest = None
    resultJR = self.jobReport.generateForwardDISET()
    if not resultJR['OK']:
      LOG.warn("Could not generate Operation for job report with result:\n%s" % (resultJR))
    else:
      reportRequest = resultJR['Value']
    if reportRequest:
      LOG.info("Populating request with job report information")
      request.addOperation(reportRequest)

    accountingReport = self.workflow_commons.get('AccountingReport', None)
    if accountingReport:
      resultAR = accountingReport.commit()
      if not resultAR['OK']:
        LOG.error("!!! Both Accounting and RequestDB are down? !!!")
        return resultAR

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      if 'ProductionOutputData' in self.workflow_commons:
        prodOutputLFNs = self.workflow_commons['ProductionOutputData'].split(";")
        LOG.info("There was some kind of error, cleaning up outputData: %s" % prodOutputLFNs)
        self.setApplicationStatus("Creating Removal Requests")
        self._cleanUp(prodOutputLFNs)

    if not len(request):  # pylint: disable=len-as-condition
      LOG.info("No Requests to process ")
      return S_OK()

    isValid = RequestValidator().validate(request)
    if not isValid['OK']:
      raise RuntimeError("Failover request is not valid: %s" % isValid['Message'])

    requestJSON = request.toJSON()
    if not requestJSON['OK']:
      raise RuntimeError(requestJSON['Message'])

    LOG.info("Creating failover request for deferred operations for job %d" % int(self.jobID))
    request_string = str(requestJSON['Value'])
    LOG.debug(request_string)
    # Write out the request string
    fname = '%s_%s_request.json' % (self.productionID, self.prod_job_id)
    with open(fname, 'w') as jsonFile:
      jsonFile.write(request_string)
    LOG.info("Created file containing failover request %s" % fname)
    resultDigest = request.getDigest()
    if resultDigest['OK']:
      LOG.info("Digest of the request: %s" % resultDigest['Value'])
    else:
      LOG.error("No digest? That's not sooo important, anyway: %s" % resultDigest['Message'])

    LOG.notice("Request After: %s" % pformat(request))
    return S_OK()

  def redirectLogOutput(self, fd, message):
    """Catch the output from the application print ``message`` to stdout and to the ``self.applicationLog`` file.

    * If ``self.eventstring`` is None print everything.
    * If it is an empty list, an empty string, or an empty string in a list print nothing
    * If it is a string or a list of strings print only matching strings

    :param int fd: if 1 append message to ``self.stdError``
    :param str message: message string
    :returns: None
    """
    sys.stdout.flush()
    if not message:
      return
    if fd == 1:
      self.stdError += message

    if isinstance(self.eventstring, six.string_types):
      self.eventstring = [self.eventstring]

    if self.eventstring is None:
      print(message)

    elif self.eventstring and self.eventstring[0]:
      for mystring in self.eventstring:
        if re.search(re.escape(mystring), message):
          print(message)
          break

    if not self.applicationLog:
      LOG.error("Application Log file not defined")
      return

    with open(self.applicationLog, 'at') as log:
      if self.excludeAllButEventString and isinstance(self.eventstring, list) and self.eventstring[0]:
        for mystring in self.eventstring:
          if re.search(re.escape(mystring), message):
            log.write(message + '\n')
            break
      elif not self.excludeAllButEventString:
        log.write(message + '\n')

  def addRemovalRequests(self, lfnList):
    """Create removalRequests for lfns in lfnList and add it to the common request."""
    LOG.info("Adding removal request")
    request = self._getRequestContainer()
    remove = Operation()
    remove.Type = "RemoveFile"

    for lfn in lfnList:
      rmFile = File()
      rmFile.LFN = lfn
      remove.addFile(rmFile)

    request.addOperation(remove)
    self.workflow_commons['Request'] = request

  def logWorkingDirectory(self):
    """log the content of the working directory."""
    res = shellCall(0, 'ls -laR')
    if res['OK'] and res['Value'][0] == 0:
      LOG.info('The contents of the working directory...')
      LOG.info(str(res['Value'][1]))
    else:
      LOG.error('Failed to list the working directory', str(res['Value'][2]))

    #############################################################################
  def _cleanUp(self, lfnList):
    """Clean up uploaded data for the LFNs in the list."""
    typeList = ['RegisterFile', 'ReplicateAndRegister', 'RemoveReplica']
    request = self._getRequestContainer()

    # keep all the requests which are not in typeList or whose file is not in lfnList
    request.__operations__ = [
        op for op in request for opFile in op if op.Type not in typeList or opFile.LFN not in lfnList]

    # just in case put the request object back to common request
    self.workflow_commons['Request'] = request

    # Set removal requests just in case
    self.addRemovalRequests(lfnList)

    return S_OK()

  def treatSteeringFiles(self):
    """treat steeringfiles."""
    steeringfilevers = self.step_commons["SteeringFileVers"]
    LOG.verbose("Will get all the files from the steeringfiles%s" % steeringfilevers)
    res = getSteeringFileDir(self.platform, steeringfilevers)
    if not res['OK']:
      LOG.error("Cannot find the steering file directory: %s" % steeringfilevers,
                     res['Message'])
      return S_ERROR("Failed to locate steering files %s" % steeringfilevers)
    path = res['Value']
    list_f = os.listdir(path)
    for localFile in list_f:
      if os.path.exists("./" + localFile):
        LOG.verbose("Found local file, don't overwrite")
        # Do not overwrite local files with the same name
        continue
      try:
        if os.path.isdir(os.path.join(path, localFile)):
          shutil.copytree(os.path.join(path, localFile), "./" + localFile)
        else:
          shutil.copy2(os.path.join(path, localFile), "./" + localFile)
      except EnvironmentError as why:
        LOG.error('Could not copy %s here because :' % localFile, str(why))
    return S_OK()

  def treatConfigPackage(self):
    """treat the config software package.

    If a 'ConfigPackage' is part of the workflow_dictionary check for the
    package and copy the steering files to the working directory. Existing files
    are _not_ overwritten.
    """

    # Find if there is a configPackage
    configName = None
    for key in self.workflow_commons:
      if 'ConfigPackage' in key:
        configName = key
        break
    if configName is None:
      LOG.verbose("No ConfigPackage set")
      return S_OK()

    configType = configName.split('ConfigPackage')[0]
    config_dir = self.workflow_commons[configName]
    packageName = str(configType + 'Config').lower()
    configVersion = config_dir.split('Config')[1]
    resCVMFS = checkCVMFS(self.platform, (packageName, configVersion))
    if resCVMFS['OK']:
      configPath = resCVMFS['Value'][0]
    else:
      LOG.error("Cannot find %sConfig on CVMFS" % configType, ("Version: " + config_dir, resCVMFS['Message']))
      resLoc = getSoftwareFolder(self.platform, packageName, configVersion)
      if resLoc['OK']:
        configPath = resLoc['Value']
      else:
        LOG.error("Cannot find %s" % config_dir, resLoc['Message'])
        return S_ERROR('Failed to locate %s as config dir' % config_dir)

    LOG.info("Found %sConfig here:" % configType, configPath)

    list_f = os.listdir(configPath)
    for localFile in list_f:
      if os.path.exists("./" + localFile):
        LOG.verbose("Found local file, don't overwrite:", localFile)
        # Do not overwrite local files with the same name
        continue
      try:
        if os.path.isdir(os.path.join(configPath, localFile)):
          shutil.copytree(os.path.join(configPath, localFile), "./" + localFile)
        else:
          shutil.copy2(os.path.join(configPath, localFile), "./" + localFile)
      except EnvironmentError as why:
        LOG.error('Could not copy %s here because %s!' % (localFile, str(why)))
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Executor to check which Sites have the proper software installed for any given job.

Based on some part of the SoftwarePackage name we ban a lists of sites.

Arbitrary number of BanLists can be created. In the CS: BanLists is a list of strings, for each string create two more
options.

* <string>Reason
* <string>Sites

Where Reason is the substring of the softwarePackage that is looked for and Sites is a lists of sites to be banned if
the software package includes the substring

Example:

+----------------------------+--------------------------------------------+---------------------------------------+
|  **Option**                |    **Description**                         |  **Example**                          |
+----------------------------+--------------------------------------------+---------------------------------------+
| BanLists                   | List of Reason/Sites combinations          | BanLists=CVMFS                        |
|                            |                                            |                                       |
+----------------------------+--------------------------------------------+---------------------------------------+
| CVMFSReason                | String matched in software version names   | CVMFSReason=ILCSoft                   |
+----------------------------+--------------------------------------------+---------------------------------------+
| CVMFSSites                 | Sites added to the ban lists of the        | CVMFSSites=LCG.SomeSite.cern          |
|                            | software version matches                   |                                       |
+----------------------------+--------------------------------------------+---------------------------------------+
"""

from __future__ import absolute_import
import six
__RCSID__ = '$Id$'

import time
from pprint import pformat

from DIRAC import S_OK
from DIRAC.WorkloadManagementSystem.Executor.Base.OptimizerExecutor import OptimizerExecutor


class SoftwareVersions(OptimizerExecutor):
  """Executor Class for Auto Banning based on software use."""

  @classmethod
  def initializeOptimizer(cls):
    """Initialize specific parameters for SoftwareVersions."""
    cls.__softToBanned = {}
    cls.__lastCacheUpdate = 0
    cls.__cacheLifeTime = 600
    return S_OK()

  def _updateBanLists(self):
    """Update the list of banned sites and reasons."""
    banLists = self.ex_getOption('BanLists', [])
    self.log.notice(pformat(banLists))

    for banList in banLists:
      resReason = self.ex_getOption(banList + 'Reason', '')
      resSites = self.ex_getOption(banList + 'Sites', [])
      self.__softToBanned[resReason] = resSites

    self.log.notice('BanLists:', pformat(self.__softToBanned))

    return S_OK()

  def optimizeJob(self, jid, jobState):
    """Update the banlists if pattern matched."""
    now = time.time()
    if (now - self.__lastCacheUpdate) > self.__cacheLifeTime:
      self.__lastCacheUpdate = now
      self._updateBanLists()

    result = jobState.getManifest()
    if not result['OK']:
      return result
    manifest = result['Value']

    software = manifest.getOption('SoftwarePackages')
    self.log.verbose('SoftwarePackages: %s ' % software)
    if isinstance(software, six.string_types):
      software = [software]

    if software:
      self.checkSoftware(manifest, software)

    result = jobState.setStatus('SoftwareCheck',
                                'Done',
                                appStatus='',
                                source=self.ex_optimizerName())
    if not result['OK']:
      return result

    self.log.verbose('Done SoftwareVersioning')

    return self.setNextOptimizer(jobState)

  def checkSoftware(self, manifest, software):
    """Check Manifest and update BannedSites.

    Check if there are softwarepackages needed for the job and ban all sites if there is some prohibitions for that
    package.
    """
    bannedSites = manifest.getOption('BannedSites', [])
    if not bannedSites:
      bannedSites = manifest.getOption('BannedSite', [])

    self.log.verbose('Original BannedSites:', bannedSites)

    softBanned = set()
    for reason, sites in self.__softToBanned.items():
      for package in software:
        self.log.verbose('Checking %s against %s ' % (reason, package))
        if reason in package:
          softBanned.update(sites)

    newBannedSites = sorted(set(bannedSites).union(softBanned))
    newBannedSites = ', '.join(newBannedSites)
    manifest.setOption('BannedSites', newBannedSites)

    self.log.notice('Updated BannedSites', newBannedSites)
    return



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the dirac-ilc-moving-transformation script."""

from __future__ import absolute_import
import unittest
import importlib
import pytest

from mock import MagicMock as Mock, patch

from DIRAC import S_OK, S_ERROR

from ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters import Params, checkDatatype
THE_SCRIPT = 'ILCDIRAC.ILCTransformationSystem.scripts.dirac_ilc_moving_transformation'
THE_REPL_SCRIPT = 'ILCDIRAC.ILCTransformationSystem.scripts.dirac_ilc_replication_transformation'

__RCSID__ = "$Id$"

PARAMS = dict(flavour='Moving')


class MyParams(Params):
  """Replacement to set some parameters for Param class."""

  def __init__(self):
    self.metaKey = 'MyKey'
    self.forcemoving = True
    self.errorMessages = []
    self.groupName = ''
    self.extraname = ''
    self.enable = False
    for name, val in PARAMS.items():
      setattr(self, name, val)


def getProxyMock(success=True):
  """return value for getProxy."""
  if success:
    return Mock(return_value=S_OK({'groupProperties': ['ProductionManagement']}))

  return Mock(return_value=S_ERROR("Failed"))


def OpMock(fail=False):
  """return mock for config operations."""
  opMock = Mock()
  if fail:
    opMock.getOptionsDict.return_value = S_ERROR('Nope')
  else:
    opMock.getOptionsDict.return_value = S_OK({'REC': 'MCReconstruction_Overlay',
                                               'Gen': 'MCGeneration'})
  return Mock(return_value=opMock)


class TestMovingParams(unittest.TestCase):
  """Test the parameters for the moving creation script."""

  def setUp(self):
    self.arguments = []
    self.sMock = Mock()
    self.sMock.getPositionalArgs.return_value = self.arguments
    self.params = Params()

  def tearDown(self):
    pass

  @patch("DIRAC.TransformationSystem.Utilities.ReplicationCLIParameters.getVOMSVOForGroup",
         new=Mock(return_value="VOMSED_VO"))
  @patch("DIRAC.TransformationSystem.Utilities.ReplicationCLIParameters.getProxyInfo",
         new=getProxyMock())
  def test_checkSettings(self):
    self.arguments = ['12345', "TargetSE", "SourceSE", "GEN"]
    self.sMock.getPositionalArgs.return_value = self.arguments
    ret = self.params.checkSettings(self.sMock)
    self.assertTrue(ret['OK'], ret.get("Message", ""))
    self.assertEqual(self.params.metaValues, [12345])
    self.assertEqual(self.params.sourceSE, ["SourceSE"])
    self.assertEqual(self.params.targetSE, ["TargetSE"])
    self.assertEqual(self.params.datatype, "GEN")

  def test_checkSettings_FailData(self):
    self.arguments = ['12345', "TargetSE", "SourceSE", "DNA"]
    self.sMock.getPositionalArgs.return_value = self.arguments
    ret = self.params.checkSettings(self.sMock)
    self.assertFalse(ret['OK'], str(ret))
    self.assertTrue(any("ERROR: Unknown Datatype" in msg for msg in self.params.errorMessages))

  def test_checkSettings_FailArgumentSize(self):
    self.arguments = ['12345', "TargetSE", "SourceSE"]
    self.sMock.getPositionalArgs.return_value = self.arguments
    ret = self.params.checkSettings(self.sMock)
    self.assertFalse(ret['OK'], str(ret))
    self.assertTrue(any("ERROR: Not enough arguments" in msg for msg in self.params.errorMessages))

  @patch("DIRAC.TransformationSystem.Utilities.ReplicationCLIParameters.getProxyInfo",
         new=getProxyMock(False))
  def test_FailProxy(self):
    self.arguments = ['12345', "TargetSE", "SourceSE", "GEN"]
    self.sMock.getPositionalArgs.return_value = self.arguments
    ret = self.params.checkSettings(self.sMock)
    self.assertFalse(ret['OK'], str(ret))
    self.assertTrue(any("ERROR: No Proxy" in msg for msg in self.params.errorMessages), str(self.params.errorMessages))

  def test_setExtraName(self):
    ret = self.params.setExtraname("extraName")
    self.assertTrue(ret['OK'], ret.get('Message', ""))
    self.assertEqual("extraName", self.params.extraname)

  def test_checkDatatype(self):
    tMock = Mock()
    tMock.getTransformations.return_value = S_OK([dict(Type="MCReconstruction_Overlay")])
    module_name = "ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters"
    with patch(module_name + ".TransformationClient", new=Mock(return_value=tMock)), \
         patch(module_name + ".Operations", new=OpMock()):
      ret = checkDatatype(123, "REC")
      self.assertTrue(ret['OK'], ret.get('Message', ""))

  @patch("ILCDIRAC.Core.Utilities.CheckAndGetProdProxy.checkAndGetProdProxy", new=Mock(return_value=S_OK()))
  def test_checkDatatype_fail0(self):
    tMock = Mock()
    tMock.getTransformations.return_value = S_OK([dict(Type="MCReconstruction_Overlay")])
    module_name = "ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters"
    with patch(module_name + ".TransformationClient", new=Mock(return_value=tMock)), \
         patch(module_name + ".Operations", new=OpMock()):
      ret = checkDatatype(123, "Gen")
      self.assertFalse(ret['OK'], ret.get('Message', ""))

  @patch("ILCDIRAC.Core.Utilities.CheckAndGetProdProxy.checkAndGetProdProxy", new=Mock(return_value=S_OK()))
  def test_checkDatatype_fail1(self):
    tMock = Mock()
    tMock.getTransformations.return_value = S_OK([dict(Type="MCReconstruction_Overlay"),
                                                    dict(Type="MCReconstruction_Overlay"),
                                                 ])
    module_name = "ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters"
    with patch(module_name + ".TransformationClient", new=Mock(return_value=tMock)):
      ret = checkDatatype(123, "REC")
      self.assertFalse(ret['OK'], ret.get('Message', ""))

  @patch("ILCDIRAC.Core.Utilities.CheckAndGetProdProxy.checkAndGetProdProxy", new=Mock(return_value=S_OK()))
  def test_checkDatatype_fail2(self):
    tMock = Mock()
    tMock.getTransformations.return_value = S_ERROR('Failed to do something')
    module_name = "ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters"
    with patch(module_name + ".TransformationClient", new=Mock(return_value=tMock)):
      ret = checkDatatype(123, "REC")
      self.assertFalse(ret['OK'], ret.get('Message', ""))

  @patch("ILCDIRAC.Core.Utilities.CheckAndGetProdProxy.checkAndGetProdProxy", new=Mock(return_value=S_OK()))
  def test_checkDatatype_fail3(self):
    """Fail checkDataType when getting Operations config."""
    tMock = Mock()
    tMock.getTransformations.return_value = S_OK([dict(Type="MCReconstruction_Overlay")])
    module_name = "ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters"
    with patch(module_name + ".TransformationClient", new=Mock(return_value=tMock)), \
         patch(module_name + ".Operations", new=OpMock(fail=True)):
      ret = checkDatatype(123, "REC")
      self.assertFalse(ret['OK'], ret.get('Message', ""))
      assert ret['Message'] == 'Nope'

@pytest.fixture
def mockProxy(mocker):
  mocker.patch('DIRAC.TransformationSystem.Utilities.ReplicationCLIParameters.getProxyInfo', new=getProxyMock())
  mocker.patch('DIRAC.TransformationSystem.Utilities.ReplicationCLIParameters.getVOMSVOForGroup',
               new=Mock(return_value='VOMSED_VO'))
  return


@pytest.fixture
def movingModule(mockProxy):
  """Fixture for the script module, mocking some parts."""
  theScript = importlib.import_module(THE_SCRIPT)
  theScript.Script = Mock(name='ScriptMock')
  theScript.Script.parseCommandLine = Mock(name='pclMock')
  theScript.getTransformationGroup = Mock(return_value='someGroup')
  theScript.checkDatatype = Mock(return_value=S_OK('SIM'))
  theScript.createDataTransformation = Mock(return_value=S_OK())
  return theScript


@pytest.fixture
def replModule(mockProxy):
  """Fixture for the script module, mocking some parts."""
  theScript = importlib.import_module(THE_REPL_SCRIPT)
  theScript.Script = Mock(name='ScriptMock')
  theScript.Script.parseCommandLine = Mock(name='pclMock')
  theScript.getTransformationGroup = Mock(return_value='someGroup')
  theScript.checkDatatype = Mock(return_value=S_OK('SIM'))
  theScript.createDataTransformation = Mock(return_value=S_OK())
  return theScript


def test_movingmain(movingModule, mocker):
  movingModule.Script.getPositionalArgs.return_value = ['12345', 'TargetSE', 'SourceSE', 'SIM']
  parDict = dict(flavour='Moving',
                 targetSE=['TargetSE'],
                 sourceSE=['SourceSE'],
                 metaKey='FooID',
                 metaValue=12345,
                 extraData={'Datatype': 'SIM'},
                 extraname='',
                 plugin='BroadcastProcessed',
                 groupSize=10,
                 tGroup='someGroup',
                 enable=False,
               )
  mocker.patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue',
               new=Mock(return_value='FooID'))
  assert movingModule.main() == 0
  movingModule.createDataTransformation.assert_called_with(**parDict)


def test_movingmain_force(movingModule, mocker):
  movingModule.Script.getPositionalArgs.return_value = ['12345', 'TargetSE', 'SourceSE', 'SIM']
  PARAMS['flavour'] = 'Moving'
  PARAMS['forcemoving'] = True
  mocker.patch(THE_SCRIPT + '._Params', spec='ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters.Params',
               new=MyParams)
  parDict = dict(flavour='Moving',
                 targetSE=['TargetSE'],
                 sourceSE=['SourceSE'],
                 metaKey='MyKey',
                 metaValue=12345,
                 extraData={'Datatype': 'SIM'},
                 extraname='',
                 plugin='Broadcast',
                 groupSize=10,
                 tGroup='someGroup',
                 enable=False,
               )
  assert movingModule.main() == 0
  movingModule.createDataTransformation.assert_called_with(**parDict)


def test_movingmain_REC(movingModule, mocker):
  movingModule.Script.getPositionalArgs.return_value = ['12346', 'TargetSE', 'SourceSE', 'REC']
  PARAMS['flavour'] = 'Moving'
  PARAMS['forcemoving'] = False
  mocker.patch(THE_SCRIPT + '._Params', spec='ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters.Params',
               new=MyParams)
  parDict = dict(flavour='Moving',
                 targetSE=['TargetSE'],
                 sourceSE=['SourceSE'],
                 metaKey='MyKey',
                 metaValue=12346,
                 extraData={'Datatype': 'REC'},
                 extraname='',
                 plugin='Broadcast',
                 groupSize=10,
                 tGroup='someGroup',
                 enable=False,
               )
  assert movingModule.main() == 0
  movingModule.createDataTransformation.assert_called_with(**parDict)


def test_movingmain_dtypeFail(movingModule):
  movingModule.Script.getPositionalArgs.return_value = ['12345', 'TargetSE', 'SourceSE', 'REC']
  movingModule.checkDatatype = Mock(name='CheckDType', return_value=S_ERROR())
  assert movingModule.main() == 1
  movingModule.checkDatatype.assert_called_once()


def test_movingmain_createFail(movingModule):
  movingModule.Script.getPositionalArgs.return_value = ['12345', 'TargetSE', 'SourceSE', 'SIM']
  movingModule.createDataTransformation = Mock(name='createT', return_value=S_ERROR())
  assert movingModule.main() == 1
  movingModule.createDataTransformation.assert_called_once()


def test_movingmain_notEnougParameters(movingModule):
  movingModule.Script.getPositionalArgs.return_value = ['12345', 'TargetSE', 'SourceSE']
  assert movingModule.main() == 1


def test_replmain(replModule, mocker):
  replModule.Script.getPositionalArgs.return_value = ['12345', 'TargetSE', 'SourceSE', 'SIM']
  mocker.patch(THE_REPL_SCRIPT + '._Params', spec='ILCDIRAC.ILCTransformationSystem.Utilities.DataParameters.Params',
                        new=MyParams)
  PARAMS['flavour'] = 'Replication'
  PARAMS['plugin'] = 'Broadcast'
  parDict = dict(flavour='Replication',
                 targetSE=['TargetSE'],
                 sourceSE=['SourceSE'],
                 metaKey='MyKey',
                 metaValue=12345,
                 extraData={'Datatype': 'SIM'},
                 extraname='',
                 plugin='Broadcast',
                 groupSize=10,
                 tGroup='someGroup',
                 enable=False,
               )
  assert replModule.main() == 0
  replModule.createDataTransformation.assert_called_with(**parDict)


def test_replmain_createFail(replModule):
  replModule.Script.getPositionalArgs.return_value = ['12345', 'TargetSE', 'SourceSE', 'SIM']
  replModule.createDataTransformation = Mock(name='createT', return_value=S_ERROR())
  assert replModule.main() == 1
  replModule.createDataTransformation.assert_called_once()


def test_replmain_notEnougParameters(replModule):
  assert replModule.main() == 1



"""Retrieve MIP peak ratios (Ring/Other to EndCap) for HCal from Calibration.txt file."""

from __future__ import print_function
from __future__ import absolute_import
import sys
from Helper_Functions import find_between


Calibration_File_And_Path = sys.argv[1]
Energy_To_Calibrate = sys.argv[2]

# Mean is the last value for mean of Gaussian fit written to the Calibration.txt file.

Ring_MIP_Peak = ''
EndCap_MIP_Peak = ''

with open(Calibration_File_And_Path, 'r') as f:
    searchlines = f.readlines()
    for line in searchlines:
        if 'HCal Ring MIP Peak' in line:
            Ring_MIP_Peak = float(find_between(line, ' : ', ' :'))
        if 'HCal EndCap MIP Peak' in line:
            EndCap_MIP_Peak = float(find_between(line, ' : ', ' :'))

Calibration_Text = '_____________________________________________________________________________________' + '\n'

Calibration_Text += 'HCal_Ring_Digi_Extract.py finding MIP peak ratios (Ring/Other to EndCap) for HCal' + '\n'

Calibration_Text += 'For Muons with energy                              :' + str(Energy_To_Calibrate) + ' /GeV \n\n'

Calibration_Text += 'Ring MIP Peak                                      :' + str(Ring_MIP_Peak) + ' /ADC Units \n\n'

Calibration_Text += 'EndCap MIP Peak                                    :' + str(EndCap_MIP_Peak) + ' /ADC Units \n\n'

with open(Calibration_File_And_Path, 'a') as myfile:
    myfile.write(Calibration_Text)

print(str(float(EndCap_MIP_Peak) / float(Ring_MIP_Peak)))



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Reports any applications errors at the end of the Workflow execution.

Depends on the 'ErrorDict' of the workflow_commons

:since: June 11, 2018
:author: A. Sailer
"""

from __future__ import absolute_import
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from DIRAC import S_OK, gLogger

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class ReportErrors(ModuleBase):
  """Reports errors from applications at the end of the workflow execution."""

  def __init__(self):
    """Constructor, no arguments."""
    super(ReportErrors, self).__init__()
    self.result = S_OK()

  def execute(self):
    """Print out the errors from all applications.

    ErrorDict is filled in
    :func:`ILCDIRAC.Workflow.Modules.ModuleBase.ModuleBase.finalStatusReport`, which is called from
    all modules.
    """
    errorDict = self.workflow_commons.get('ErrorDict', {})
    if not errorDict:
      LOG.info("No errors encountered")

    for app, errorMessages in errorDict.items():
      for message in errorMessages:
        LOG.error(app, message)
    return S_OK()
